"""A python script for working with amazon S3"""
import json
import os
from time import sleep

import boto3
import botocore
import urllib3
from botocore.exceptions import ClientError

from common_libraries.decorator import Decorator
from common_libraries.logger import TestLog


class Boto3Client:
    s3_client = None
    s3_resource = None
    delay_time = int(os.getenv("vcd-site_delay_time", default=2))
    max_attempts = int(os.getenv("vcd-site_max_attempts", default=5))
    AWS_REGION = "us-east-1"

    # disable ssl warning
    urllib3.disable_warnings()

    def __init__(self):
        self.logger = TestLog().set_logger(log_path=os.environ["test_log_file"],
                                           logging_level=os.environ["logging_level"])

    def __init__(self, access_key=None, secret_key=None,
                 region_name="default", endpoint=None,
                 connection_type="client"):
        """
        Class Init
        :param access_key: user's access key
        :param secret_key: user's secret key
        :param region_name: platform's region
        :param endpoint: the endpoint used for testing
        :param connection_type: boto3 supports two different types: client and source
        """
        if "vcd-site_default_region" in os.environ and os.environ["vcd-site_default_region"] != "None":
            region_name = os.environ["vcd-site_default_region"]
        else:
            os.environ["vcd-site_default_region"] = region_name
        self.logger = TestLog().set_logger(log_path=os.environ["test_log_file"],
                                           logging_level=os.environ["logging_level"])
        print("default region-name: " + region_name)
        if access_key is None:
            # by default, use the user's access key and secret key
            endpoint = os.environ['vcd-site_ose_url']
            access_key = eval(os.environ['vcd-site_ossauto1_tu1'])['access_key']
            secret_key = eval(os.environ['vcd-site_ossauto1_tu1'])['secret_key']
        params = {'aws_access_key_id': access_key,
                  'aws_secret_access_key': secret_key, 'verify': False, 'use_ssl': False,
                  'region_name': region_name, 'endpoint_url': endpoint}
        if connection_type == "client":
            boto3_client = boto3.client(service_name="s3", **params)
            boto3_resource = boto3.resource(service_name="s3", **params)
            self.s3_client = boto3_client
            self.s3_resource = boto3_resource
            self.logger.debug("======> boto3 client initialize with parameters: " + str(params) + " <======")
        elif connection_type == 'resource':
            boto3_client = boto3.client(service_name="s3", **params)
            boto3_resource = boto3.resource(service_name="s3", **params)
            self.s3_client = boto3_client
            self.s3_resource = boto3_resource
            self.logger.debug("======> boto3 client initialize with parameters: " + str(params) + " <======")

    def get_boto3_client(self, endpoint, access_key, secret_key, region_name="default"):
        """
        Get boto3 client instance
        :param endpoint:
        :param access_key:
        :param secret_key:
        :param region_name:
        :return:
        """
        if "vcd-site_default_region" in os.environ and os.environ["vcd-site_default_region"] != "None":
            region_name = os.environ["vcd-site_default_region"]
        else:
            os.environ["vcd-site_default_region"] = region_name
        params = {'aws_access_key_id': access_key,
                  'aws_secret_access_key': secret_key, 'verify': False, 'use_ssl': False,
                  'region_name': region_name, 'endpoint_url': endpoint}
        # basic config
        config = botocore.config.Config(
            retries=dict(
                max_attempts=self.max_attempts
            )
        )
        print("boto3 region-name: " + region_name)
        # Initialize the S3 client
        boto3_client = boto3.client(service_name="s3", config=config, **params)
        self.s3_client = boto3_client
        return boto3_client

    def get_boto3_resource(self, endpoint, access_key, secret_key, region_name="default"):
        """
        Get boto 3 resource instance
        :param endpoint:
        :param access_key:
        :param secret_key:
        :param region_name:
        :return:
        """
        if "vcd-site_default_region" in os.environ and os.environ["vcd-site_default_region"] != "None":
            region_name = os.environ["vcd-site_default_region"]
        else:
            os.environ["vcd-site_default_region"] = region_name
        params = {'aws_access_key_id': access_key,
                  'aws_secret_access_key': secret_key, 'verify': False, 'use_ssl': False,
                  'region_name': region_name, 'endpoint_url': endpoint}
        config = botocore.config.Config(
            retries=dict(
                max_attempts=self.max_attempts
            )
        )
        print("boto3 region-name: " + region_name)
        boto3_resource = boto3.resource(service_name="s3", config=config, **params)
        return boto3_resource

    # boto3 common methods
    @Decorator.log(my_logger=TestLog())
    @Decorator.exception_handler()
    def get_list_of_buckets(self):
        """
        Returns a list of all buckets owned by the authenticated sender of the request
        :return:
        """
        buckets_list = []
        self.logger.debug("Buckets:")
        for bucket in self.s3_client.list_buckets()['Buckets']:
            self.logger.debug(bucket['Name'])
            buckets_list.append(bucket['Name'])
        return buckets_list

    @Decorator.log(my_logger=TestLog())
    @Decorator.exception_handler()
    def s3_delete_all_objects(self, bucket_name):
        log_message = "Cleaning all objects form bucket: " + bucket_name
        delete_object_response = None
        self.logger.debug(log_message)
        try:
            if self.s3_waiters_check_bucket_exists(bucket_name=bucket_name):
                bucket = self.s3_resource.Bucket(bucket_name)
                bucket.object_versions.all().delete()
                TestLog.attach_allure_log(
                    log_message=log_message,
                    log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
                if delete_object_response:
                    return delete_object_response
                else:
                    log_message = "No object(s) found for the bucket: " + bucket_name
                    TestLog.attach_allure_log(
                        log_message=log_message,
                        log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
                    self.logger.debug("No object(s) found for the bucket: " + bucket_name)
                    return True
        except botocore.exceptions.ClientError as ce:
            self._log_error(bucket_name=bucket_name, exception_info=ce)

    @Decorator.log(my_logger=TestLog())
    @Decorator.exception_handler()
    def s3_check_if_file_exist(self, bucket_name, file):
        log_message = "Checking if file '" + file + "' exists in bucket '" + bucket_name + "'"
        self.logger.debug(log_message)
        try:
            self.s3_client.head_object(Bucket=bucket_name, Key=file)
            TestLog.attach_allure_log(
                log_message=log_message,
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        except ClientError as e:
            return int(e.response['Error']['Code']) != 404
        return True

    @Decorator.log(my_logger=TestLog())
    @Decorator.exception_handler()
    def s3_waiters_check_object_exists(self, bucket_name, key_name):
        try:
            waiter = self.s3_client.get_waiter('object_exists')
            waiter.wait(Bucket=bucket_name, Key=key_name,
                        WaiterConfig={
                            'Delay': self.delay_time, 'MaxAttempts': self.max_attempts})
            TestLog.attach_allure_log(
                log_message='Object exists: ' + bucket_name + '/' + key_name,
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            return True
        except ClientError as ce:
            self._log_message(bucket_name=bucket_name, exception_info=ce)

    @Decorator.log(my_logger=TestLog())
    @Decorator.exception_handler()
    def s3_waiters_check_bucket_exists(self, bucket_name):
        try:
            waiter = self.s3_client.get_waiter('bucket_exists')
            waiter.wait(Bucket=bucket_name,
                        WaiterConfig={
                            'Delay': self.delay_time, 'MaxAttempts': self.max_attempts})
            TestLog.attach_allure_log(
                log_message="Bucket '" + bucket_name + "' found as expected",
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            return True
        except ClientError as ce:
            self._log_message(bucket_name=bucket_name, exception_info=ce)
            return False

    @Decorator.log(my_logger=TestLog())
    @Decorator.exception_handler()
    def s3_create_bucket(self, bucket_name, region_override=None, acl=None, grant_acl=None, region_name="default"):
        """Create an S3 bucket in a specified region
        If a region is not specified, the bucket is created in the S3 default
        region (us-east-1).
        :param region_override: AWS region where to create the bucket.
        :param grant_acl: GrantFullControl/GrantRead/GrantReadACP/GrantWrite/GrantWriteACP
        :param acl: The canned ACL to apply to the bucket: 'private'|'public-read'|'public-read-write'|'authenticated-read'
        :param bucket_name: the name of bucket
        :return: True if bucket created, else False
        """
        http_status_code = None
        try:
            if self.s3_resource_check_bucket_exists(bucket_name):
                self.s3_delete_bucket(bucket_name=bucket_name)
                TestLog.attach_allure_log(
                    log_message="Bucket delete successfully : " + bucket_name,
                    log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            else:
                TestLog.attach_allure_log(
                    log_message="Bucket not exist in the system : " + bucket_name,
                    log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            self.logger.debug("======> start to create bucket: " + bucket_name)
            # create bucket with default region
            if "vcd-site_default_region" in os.environ and os.environ["vcd-site_default_region"] != "None":
                region_name = os.environ["vcd-site_default_region"]
            else:
                os.environ["vcd-site_default_region"] = region_name
            if region_name:
                location = {'LocationConstraint': region_name}
            else:
                raise Exception("Region name should be provided!")
            if region_override is None and acl is None:
                response = self.s3_client.create_bucket(Bucket=bucket_name, CreateBucketConfiguration=location)
                http_status_code = str(response['ResponseMetadata']['HTTPStatusCode'])
            # Create bucket with canned ACL
            if acl is not None:
                response = self.s3_client.create_bucket(Bucket=bucket_name, ACL=acl, CreateBucketConfiguration=location)
                self.logger.debug("create bucket with acl: " + str(response))
                http_status_code = str(response['ResponseMetadata']['HTTPStatusCode'])
            # Create bucket with Granted ACL
            if grant_acl is not None:
                parameters = eval(grant_acl)
                parameters['CreateBucketConfiguration'] = location
                parameters['Bucket'] = bucket_name
                response = self.s3_client.create_bucket(**parameters)
                http_status_code = str(response['ResponseMetadata']['HTTPStatusCode'])
                self.logger.debug("create bucket with granted acl: " + str(response))
            self.logger.info("======> Default LocationConstraint: " + str(location))
            # Create bucket with specified region
            if region_override is not None:
                location = {'LocationConstraint': region_override}
                self.logger.debug("======> Override LocationConstraint: " + str(location))
                response = self.s3_client.create_bucket(Bucket=bucket_name, CreateBucketConfiguration=location)
                self.logger.debug("create bucket with location constraint: " + str(response))
                http_status_code = str(response['ResponseMetadata']['HTTPStatusCode'])
            # Assert bucket creation status
            if http_status_code:
                assert int(http_status_code) == 200
                # Wait for bucket creation complete
                self.s3_waiters_check_bucket_exists(bucket_name=bucket_name)
                TestLog.attach_allure_log(
                    log_message="Bucket created successfully : " + bucket_name + "and http status code is " + str(
                        http_status_code),
                    log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
                self.logger.debug("end to create bucket <======")
                return http_status_code
            else:
                raise Exception("Fail to create the Bucket '" + bucket_name + "'")
        except ClientError as ce:
            self._log_message(bucket_name=bucket_name, exception_info=ce)

    @Decorator.log(my_logger=TestLog())
    @Decorator.exception_handler()
    def s3_delete_bucket(self, bucket_name):
        """

        :param bucket_name:
        :return:
        """
        http_status_code = None
        try:
            s3_bucket = self.s3_resource.Bucket(bucket_name)
            bucket_versioning = self.s3_resource.BucketVersioning(bucket_name)
            if bucket_versioning.status == 'Enabled':
                s3_bucket.object_versions.delete()
                TestLog.attach_allure_log(
                    log_message="Object deleted with version enabled : " + bucket_name,
                    log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            else:
                s3_bucket.objects.all().delete()
                TestLog.attach_allure_log(
                    log_message="Object deleted with version suspended : " + bucket_name,
                    log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            # delete the Bucket
            delete_bucket_response = self.s3_client.delete_bucket(Bucket=bucket_name)
            TestLog.attach_allure_log(
                log_message="Bucket deleted: " + bucket_name + " And get the response " + str(
                    delete_bucket_response),
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            if delete_bucket_response:
                http_status_code = str(delete_bucket_response['ResponseMetadata']['HTTPStatusCode'])
                TestLog.attach_allure_log(
                    log_message="Bucket deleted successfully : " + bucket_name + " And get the response " + str(
                        delete_bucket_response),
                    log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
                TestLog.attach_allure_log(
                    log_message="Bucket deleted successfully : " + bucket_name + " And status code is " + str(
                        http_status_code),
                    log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            else:
                TestLog.attach_allure_log(
                    log_message="Bucket deleted failed : " + bucket_name,
                    log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
            # Assert delete bucket status
            if http_status_code:
                assert int(http_status_code) == 204
            return http_status_code
        except ClientError as ex:
            TestLog.attach_allure_log(
                log_message="Bucket deleted failed : " + bucket_name + "  And get the exception: " + str(ex),
                log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
            self.logger.error("Couldn't delete bucket and caused by exception: " + str(ex))
            return 204

    def s3_delete_buckets(self, current_bucket_name=None):
        try:
            for bucket in self.s3_resource.buckets.all():
                try:
                    bucket_name = bucket.name
                    if current_bucket_name and current_bucket_name == bucket_name:
                        self.logger.debug(
                            "current bucket '" + bucket_name + "' is used for testing so ignore the deleting")
                    else:
                        self.logger.info("=====> start to delete bucket: " + str(bucket_name))
                        s3_bucket = self.s3_resource.Bucket(bucket_name)
                        bucket_versioning = self.s3_resource.BucketVersioning(bucket_name)
                        if bucket_versioning and bucket_versioning.status == "Enabled":
                            self.logger.info("delete bucket with versioning enabled")
                            s3_bucket.object_versions.delete()
                        else:
                            self.logger.info("delete bucket and object(s)")
                            s3_bucket.objects.all().delete()
                        s3_bucket.delete()
                        self.logger.info("<===== end to delete bucket: " + str(bucket_name))
                except ClientError as ex:
                    self.logger.debug("Couldn't delete bucket and caused by exception: " + str(ex))
                    # raise Exception(ex)
        except ClientError as ex:
            self.logger.debug("Couldn't delete bucket and caused by exception: " + str(ex))

    @Decorator.log(my_logger=TestLog())
    @Decorator.exception_handler()
    def put_bucket_policy(self, bucket_name, bucket_policy):
        """
        Applies an Amazon S3 bucket policy to an Amazon S3 bucket. If you are using an identity other than the root user of the Amazon Web Services account that owns the bucket,
        the calling identity must have the PutBucketPolicy permissions on the specified bucket and belong to the bucket owner's account in order to use this operation.
        :param bucket_name: The name of the bucket.
        :param bucket_policy: The bucket policy as a JSON document.
        :return:
        """
        try:
            self.logger.debug("start to create bucket policy")
            # Convert the policy from JSON dict to string
            policy = json.dumps(bucket_policy)
            # Set bucket policy
            response = self.s3_client.put_bucket_policy(Bucket=bucket_name, Policy=policy)
            http_status_code = str(response['ResponseMetadata']['HTTPStatusCode'])
            self.logger.debug("end to create bucket policy: " + str(response))
            TestLog.attach_allure_log(
                log_message="Put Bucket Policy successfully : " + bucket_name,
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            if http_status_code:
                return http_status_code
        except ClientError as ce:
            self._log_error(bucket_name=bucket_name, exception_info=ce)

    @Decorator.log(my_logger=TestLog())
    @Decorator.exception_handler()
    def put_bucket_cors(self, bucket_name, bucket_cors):
        """
        Sets the cors configuration for your bucket. If the configuration exists, Amazon S3 replaces it.
        :param bucket_name: The name of the bucket.
        :param bucket_cors: A set of origins and methods (cross-origin access that you want to allow). You can add up to 100 rules to the configuration.
        :return:
        """
        try:
            self.logger.debug("======> start to add bucket cors")
            # Set the new bucket cors
            response = self.s3_client.put_bucket_cors(Bucket=bucket_name, CORSConfiguration=bucket_cors)
            http_status_code = str(response['ResponseMetadata']['HTTPStatusCode'])
            self.logger.debug("end to add bucket cors: " + str(response) + " =======>")
            TestLog.attach_allure_log(
                log_message="Put Bucket CORS successfully : " + bucket_name,
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            return http_status_code
        except ClientError as ce:
            self._log_error(bucket_name=bucket_name, exception_info=ce)

    @Decorator.log(my_logger=TestLog())
    @Decorator.exception_handler()
    def get_bucket_cors(self, bucket_name):
        """Retrieve the CORS configuration rules of an Amazon S3 bucket
        :param bucket_name: The name of the bucket.
        :return: True/delete successfully, False/delete failed
        """

        # Retrieve the CORS configuration
        try:
            response = self.s3_resource.BucketCors(bucket_name).cors_rules
            TestLog.attach_allure_log(
                log_message="Get Bucket CORS successfully : " + bucket_name,
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            self.logger.debug(str(response))
            return response
        except ClientError as ce:
            self._log_error(bucket_name=bucket_name, exception_info=ce)

    @Decorator.log(my_logger=TestLog())
    @Decorator.exception_handler()
    def delete_bucket_cors(self, bucket_name):
        """
        Delete the CORS configuration rules of an Amazon S3 bucket
        :param bucket_name: The name of the bucket.
        :return: True/delete successfully, False/delete failed
        """

        # Delete the CORS configuration
        try:
            response = self.s3_resource.BucketCors(bucket_name).delete()
            self.logger.debug(str(response))
            return response
        except ClientError as ce:
            self._log_error(bucket_name=bucket_name, exception_info=ce)

    # boto3 s3 resource
    @Decorator.log(my_logger=TestLog())
    @Decorator.exception_handler()
    def s3_resource_check_bucket_exists(self, bucket_name):
        """
        Check if the bucket exists
        :param bucket_name: The name of the bucket.
        :return:
        """
        try:
            self.s3_resource.meta.client.head_bucket(Bucket=bucket_name)
            TestLog.attach_allure_log(
                log_message="Bucket exist in the system : " + bucket_name,
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            return True
        except ClientError:
            return False

    @Decorator.log(my_logger=TestLog())
    @Decorator.exception_handler()
    def s3_resource_create_bucket(self, bucket_name, region_name="default", object_lock_enabled_for_bucket=False,
                                  **kwargs):
        """
        Specifies the Region where the bucket will be created. If you don't specify a Region, the bucket is created in the US East (N. Virginia) Region (us-east-1).
        :param object_lock_enabled_for_bucket:
        :param region_name:
        :param bucket_name: The name of the bucket.
        :return: http status code
        """
        create_bucket_response = None
        http_status_code = None
        if "vcd-site_default_region" in os.environ and os.environ["vcd-site_default_region"] != "None":
            region_name = os.environ["vcd-site_default_region"]
        else:
            os.environ["vcd-site_default_region"] = region_name
        if region_name:
            location = {'LocationConstraint': region_name}
            self.logger.debug("======> Default LocationConstraint: " + str(location))
        try:
            # check if bucket exist in the system
            if self.s3_resource_check_bucket_exists(bucket_name):
                self.s3_delete_bucket(bucket_name=bucket_name)
                TestLog.attach_allure_log(
                    log_message="Bucket delete successfully : " + bucket_name,
                    log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            else:
                TestLog.attach_allure_log(
                    log_message="Bucket not exist in the system : " + bucket_name,
                    log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            # create a new bucket
            self.logger.debug("======> start to create bucket: " + bucket_name)
            bucket = self.s3_resource.Bucket(bucket_name)
            if location:
                create_bucket_response = bucket.create(
                    CreateBucketConfiguration=location, ObjectLockEnabledForBucket=object_lock_enabled_for_bucket,
                    **kwargs)
            else:
                raise Exception("LocationConstraint should be provided when create bucket!")
            http_status_code = str(create_bucket_response['ResponseMetadata']['HTTPStatusCode'])
            bucket.load()
            self.logger.debug("create bucket status: " + str(http_status_code))
            # wait for bucket created successfully
            self.s3_waiters_check_bucket_exists(bucket_name=bucket_name)
            if http_status_code:
                TestLog.attach_allure_log(
                    log_message="Bucket created successfully : " + bucket_name + "and http status code is " + str(
                        http_status_code),
                    log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
                self.logger.debug("end to create bucket <======")
                return http_status_code
        except ClientError as ce:
            # retry to create the bucket
            if create_bucket_response:
                http_status_code = str(create_bucket_response['ResponseMetadata']['HTTPStatusCode'])
            # loop to re-create bucket
            for i in range(3):
                sleep(1)
                bucket = self.s3_resource.Bucket(bucket_name)
                create_bucket_response = bucket.create(
                    CreateBucketConfiguration=location)
                if create_bucket_response:
                    http_status_code = str(create_bucket_response['ResponseMetadata']['HTTPStatusCode'])
                    if int(http_status_code) == 200:
                        log_message = "Bucket created successfully after retry several times: " + bucket_name + "and http status code is " + str(
                            http_status_code)
                        self.logger.debug(log_message)
                        TestLog.attach_allure_log(
                            log_message=log_message,
                            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
                        break
            if http_status_code:
                assert int(http_status_code) == 200
                return http_status_code
            else:
                raise ce

    @Decorator.log(my_logger=TestLog())
    @Decorator.exception_handler()
    def s3_resource_delete_bucket(self, bucket_name):
        """
        Deletes the S3 bucket. All objects (including all object versions and delete markers) in the bucket must be deleted before the bucket itself can be deleted.
        :param bucket_name: The name of the bucket.
        :return: http status code
        """
        delete_bucket_response = None
        http_status_code = None
        try:
            # check if bucket exist in the system
            if self.s3_resource_check_bucket_exists(bucket_name):
                bucket = self.s3_resource.Bucket(bucket_name)
                delete_bucket_response = bucket.delete()
                http_status_code = str(delete_bucket_response['ResponseMetadata']['HTTPStatusCode'])
                TestLog.attach_allure_log(
                    log_message="Bucket delete successfully : " + bucket_name,
                    log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            else:
                http_status_code = 204
                TestLog.attach_allure_log(
                    log_message="Bucket not exist in the system : " + bucket_name,
                    log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            # wait for bucket delete successfully
            self.s3_waiters_check_bucket_exists(bucket_name=bucket_name)
            TestLog.attach_allure_log(
                log_message="Bucket created successfully : " + bucket_name + "and http status code is " + str(
                    http_status_code),
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            return http_status_code
        except ClientError as ce:
            self._log_message(bucket_name=bucket_name, exception_info=ce)
            # retry to create the bucket
            if delete_bucket_response:
                http_status_code = str(delete_bucket_response['ResponseMetadata']['HTTPStatusCode'])
            # loop to re-create bucket
            for i in range(3):
                bucket = self.s3_resource.Bucket(bucket_name)
                sleep(1)
                delete_bucket_response = bucket.delete()
                if delete_bucket_response:
                    http_status_code = str(delete_bucket_response['ResponseMetadata']['HTTPStatusCode'])
                    if int(http_status_code) == 204:
                        log_message = "Bucket created successfully after retry several times: " + bucket_name + "and http status code is " + str(
                            http_status_code)
                        self.logger.debug(log_message)
                        TestLog.attach_allure_log(
                            log_message=log_message,
                            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
                        break
            return http_status_code

    @Decorator.log(my_logger=TestLog())
    @Decorator.exception_handler()
    def s3_resource_delete_single_object(self, bucket_name, object_key):
        try:
            if self.s3_waiters_check_bucket_exists(bucket_name=bucket_name):
                log_message = "Permanently deleted all versions of object " + object_key
                bucket = self.s3_resource.Bucket(bucket_name)
                delete_object_response = bucket.object_versions.filter(Prefix=object_key).delete()
                self.logger.debug(log_message)
                TestLog.attach_allure_log(
                    log_message=log_message,
                    log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
                if delete_object_response:
                    return delete_object_response
                else:
                    log_message = "No object(s) found for the bucket: " + bucket_name
                    self.logger.debug("No object(s) found for the bucket: " + bucket_name)
                    TestLog.attach_allure_log(
                        log_message=log_message,
                        log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
                    return delete_object_response
        except ClientError as ce:
            self._log_error(bucket_name=bucket_name, exception_info=ce)

    def _log_error(self, bucket_name, exception_info):
        error_code = exception_info.response['Error']['Code']
        error_message = exception_info.response['Error']['Message']
        http_status_code = int(exception_info.response['ResponseMetadata']['HTTPStatusCode'])
        if 400 <= http_status_code < 500:
            if http_status_code == 403:
                error_message = "Private Bucket '" + bucket_name + "' has Forbidden Access!"
                self.logger.error(error_message)
            else:
                error_message = "Client side warning is found and Bucket '" + bucket_name + "' can't find!"
                self.logger.warning(error_message)
        if http_status_code > 500:
            error_message = "Server side warning is found and Bucket '" + bucket_name + "' can't find!"
            self.logger.error(error_message)
        TestLog.attach_allure_log(
            log_message="Bucket '" + bucket_name + "' doesn't exist in the system, caused by the exception '" + error_code + "' | '" + error_message + "'",
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        return http_status_code

    def _log_message(self, bucket_name, exception_info):
        error_code = exception_info.response['Error']['Code']
        error_message = exception_info.response['Error']['Message']
        http_status_code = int(exception_info.response['ResponseMetadata']['HTTPStatusCode'])
        if 400 <= http_status_code < 500:
            if http_status_code == 403:
                error_message = "Private Bucket '" + bucket_name + "' has Forbidden Access!"
                self.logger.debug(error_message)
            else:
                error_message = "Client side warning is found and Bucket '" + bucket_name + "' can't find!"
                self.logger.debug(error_message)
        if http_status_code > 500:
            error_message = "Server side warning is found and Bucket '" + bucket_name + "' can't find!"
            self.logger.debug(error_message)
        TestLog.attach_allure_log(
            log_message="Bucket '" + bucket_name + "' doesn't exist in the system, caused by the exception '" + error_code + "' | '" + error_message + "'",
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        return http_status_code