import functools
import logging
import os
from typing import Union

from botocore.exceptions import ClientError, ParamValidationError
from requests import RequestException, ConnectionError, Timeout

from common_libraries.logger import TestLog


class Decorator:

    @staticmethod
    def get_default_logger():
        # my_test_log_path = os.environ["test_log_file"]
        return TestLog().get_logger()

    def log(self=None, *, my_logger: Union[TestLog, logging.Logger] = None):
        def decorator_log(func):
            @functools.wraps(func)
            def wrapper(*args, **kwargs):
                try:
                    if my_logger is None:
                        logger = Decorator().get_default_logger()
                    else:
                        if isinstance(my_logger, TestLog):
                            # logger = my_logger.set_logger(log_path=os.environ["test_log_file"])
                            logger = my_logger.get_logger()
                        else:
                            logger = my_logger
                    args_repr = [repr(a) for a in args]
                    kwargs_repr = [f"{k}={v!r}" for k, v in kwargs.items()]
                    signature = ", ".join(args_repr + kwargs_repr)
                    log_info = f"Method [{func.__name__}] called with args {signature}"
                    logger.debug(log_info)
                    TestLog.attach_allure_log(
                        log_message=log_info,
                        log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
                    result = func(*args, **kwargs)
                    return result
                except ClientError as client_err:
                    error_message = f"Exception raised in {func.__name__}. exception: {str(client_err)}"
                    TestLog.attach_allure_log(
                        log_message=error_message,
                        log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
                    logger.error(error_message, exc_info=True)
                    raise ClientError(error_message)
                except Exception as ex:
                    error_message = f"Exception raised in {func.__name__}. exception: {str(ex)}"
                    TestLog.attach_allure_log(
                        log_message=error_message,
                        log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
                    logger.error(error_message, exc_info=True)
                    raise Exception(error_message)

            return wrapper

        if self is None:
            return decorator_log
        else:
            return decorator_log(self)

    def exception_handler(self=None, *, my_logger: Union[TestLog, logging.Logger] = None):
        def decorator_exception(func):
            @functools.wraps(func)
            def wrapper(*args, **kwargs):
                if my_logger is None:
                    logger = Decorator().get_default_logger()
                else:
                    if isinstance(my_logger, TestLog):
                        logger = my_logger.set_logger(log_path=os.environ["test_log_file"], log_name=func.__name__)
                    else:
                        logger = my_logger
                try:
                    result = func(*args, **kwargs)
                    return result
                except ParamValidationError as error:
                    error_message = f'Raised ParamValidationError when calling the function:{func.__name__}, the parameters you provided are incorrect: {error}'
                    logger.error(error_message, exc_info=True)
                    TestLog.attach_allure_log(
                        log_message=error_message,
                        log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
                    raise error
                except ConnectionError as ex:
                    error_message = f"Raised ConnectionError when calling the function:{func.__name__} and exception detail: {str(ex)}"
                    logger.error(error_message, exc_info=True)
                    TestLog.attach_allure_log(
                        log_message=error_message,
                        log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
                    raise ConnectionError(error_message)
                except Timeout as ex:
                    error_message = f"Raised Timeout exception when calling the function: {func.__name__} and exception detail: {str(ex)}"
                    logger.error(error_message, exc_info=True)
                    TestLog.attach_allure_log(
                        log_message=error_message,
                        log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
                    raise Timeout(error_message)
                except KeyError as ex:
                    error_message = f"Raised KeyError when calling the function: {func.__name__} and exception detail: {str(ex)}"
                    logger.error(error_message, exc_info=True)
                    TestLog.attach_allure_log(
                        log_message=error_message,
                        log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
                    raise ex
                except RequestException as ex:
                    error_message = f"Raised RequestException when calling the function: {func.__name__} and exception detail: {str(ex)}"
                    logger.error(error_message, exc_info=True)
                    TestLog.attach_allure_log(
                        log_message=error_message,
                        log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
                    raise ex
                except ClientError as ex:
                    error_code = ex.response['Error']['Code']
                    error_message = ex.response['Error']['Message']
                    http_status_code = int(ex.response['ResponseMetadata']['HTTPStatusCode'])
                    if 400 <= http_status_code < 500:
                        log_exception = "Calling method [" + func.__name__ + "] failed caused by client side error: error code '" + error_code + "' And '" + error_message + "'"
                    elif http_status_code >= 500:
                        log_exception = "Calling method [" + func.__name__ + "] failed caused by server side error: error code '" + error_code + "' And '" + error_message + "'"
                    else:
                        log_exception = "Calling method [" + func.__name__ + "] failed caused by other issue: error code '" + error_code + "' And error message '" + error_message + "'"
                    logger.error(log_exception, exc_info=True)
                    TestLog.attach_allure_log(
                        log_message=log_exception,
                        log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
                    raise ex

            return wrapper

        if self is None:
            return decorator_exception
        else:
            return decorator_exception(self)
