import os

import requests
from requests import RequestException, ConnectTimeout

from common_libraries.decorator import Decorator
from common_libraries.logger import TestLog


class HttpRequestsAPIClient:
    def __init__(self):
        self.logger = TestLog().set_logger(log_path=os.environ["test_log_file"],
                                           logging_level=os.environ["logging_level"])
        self._default_req_header = {
            'Accept': 'application/*+json;version=35.2,application/json;version=35.2'
        }

    @Decorator.log(my_logger=TestLog())
    @Decorator.exception_handler()
    def do_get_vcloud_authorization(self, req_url=None, req_header=None, verify=False):
        """

        :param req_url:
        :param req_header:
        :param verify:
        :return:
        """
        if req_header is None:
            req_header = {
                'Accept': 'application/*+json;version=35.2,application/json;version=35.2',
                'Authorization': 'Basic YWRtaW5AU3lzdGVtOnZtd2FyZQ=='
            }
        try:
            # Get x-vcloud-authorization
            response = requests.request("POST", req_url, headers=req_header, verify=verify)
            if response:
                x_vcloud_authorization = response.headers['x-vcloud-authorization']
                return x_vcloud_authorization, response.status_code
            else:
                self.logger.error("Request url: " + req_url)
                raise Exception("The http response is None and not receive successfully!")

        except requests.RequestException:
            raise requests.RequestException("Got requests exception")
        except requests.ConnectTimeout:
            raise requests.ConnectTimeout("Got connect timeout exception")

    @Decorator.log(my_logger=TestLog())
    @Decorator.exception_handler()
    def do_get_vcloud_jwt(self, req_url=None, req_header=None, verify=False):
        """

        :param req_url:
        :param req_header:
        :param verify:
        :return:
        """
        if req_header is None:
            req_header = {
                'Accept': 'application/*+json;version=35.2,application/json;version=35.2',
                'Authorization': 'Basic YWRtaW5AU3lzdGVtOnZtd2FyZQ=='
            }
        try:
            # Get x-vcloud-authorization
            response = requests.request("POST", req_url, headers=req_header, verify=verify)
            if response:
                jwt = response.headers['X-VMWARE-VCLOUD-ACCESS-TOKEN']
                return jwt, response.status_code
            else:
                self.logger.error("Request url: " + req_url)
                raise Exception("The http response is None and not receive successfully!")

        except requests.RequestException:
            raise requests.RequestException("Got requests exception")
        except requests.ConnectTimeout:
            raise requests.ConnectTimeout("Got connect timeout exception")

    @Decorator.log(my_logger=TestLog())
    @Decorator.exception_handler()
    def get_ose_version(self, req_url=None, req_header=None, verify=False):
        """

        :param req_url:
        :param req_header:
        :param verify:
        :return:
        """
        if req_header is None:
            req_header = {
                'Accept': 'application/*+json;version=35.2,application/json;version=35.2',
                'Authorization': 'Basic YWRtaW5AU3lzdGVtOnZtd2FyZQ=='
            }
        try:
            # Get x-vcloud-authorization
            response = requests.request("GET", req_url, headers=req_header, verify=verify)
            if response is not None:
                x_vcloud_authorization = response['version']
                return x_vcloud_authorization, response.status_code
            else:
                raise Exception("The http response is None and not receive successfully!")

        except requests.RequestException:
            raise requests.RequestException("Got requests exception")
        except requests.ConnectTimeout:
            raise requests.ConnectTimeout("Got connect timeout exception")

    def _prepare_http_header(self, req_header=None, access_key=None, secret_key=None):
        """

        :param req_header:
        :param access_key:
        :param secret_key:
        :return:
        """
        if req_header:
            pass
        else:
            req_header = self._default_req_header

    @staticmethod
    @Decorator.log(my_logger=TestLog())
    @Decorator.exception_handler()
    def do_head(req_url=None, files=None, req_data=None, query_params=None, auth=None, req_header=None):
        pass

    @Decorator.log(my_logger=TestLog())
    @Decorator.exception_handler()
    def do_get(self, req_url=None, req_header=None):
        """
        Send http get request
        :param req_url:
        :param req_header:
        :return:
        """
        try:
            response = requests.request("GET", req_url, headers=req_header, verify=False)
            if response:
                self._log_info(
                    "Http GET Response: " + response.text + " and status: " + str(response.status_code))
                return response
            else:
                self._log_error("The http response is None and not receive successfully!")

        except requests.RequestException:
            raise requests.RequestException("Got requests exception")
        except requests.ConnectTimeout:
            raise requests.ConnectTimeout("Got connect timeout exception")

    @Decorator.log(my_logger=TestLog())
    @Decorator.exception_handler(my_logger=TestLog())
    def do_post(self, req_url=None, req_data=None, req_header=None, verify=False):
        """

        :param req_url:
        :param req_data:
        :param req_header:
        :param verify:
        :return:
        """
        try:
            self._log_info("POST request payload: " + str(req_data))
            self._log_info("POST request url: " + req_url)
            self._log_info("POST request header: " + str(req_header))
            response = requests.request("POST", req_url, headers=req_header, data=req_data, verify=verify)
            if response is not None:
                self._log_info(
                    "Http POST Response: " + response.text + " and status: " + str(response.status_code))
                return response
            else:
                self._log_error("The http response is None and not receive successfully!")
        except RequestException as ex:
            log_error = "Send POST request and get the un-expected exception RequestException:" + str(ex)
            self._log_error(log_error)
        except ConnectTimeout as ex:
            log_error = "Send POST request and get the un-expected exception ConnectTimeout:" + str(ex)
            self._log_error(log_error)

    @Decorator.log(my_logger=TestLog())
    @Decorator.exception_handler()
    def do_put(self, req_url=None, req_data=None, req_header=None, verify=False):
        """

        :param req_url:
        :param req_data:
        :param req_header:
        :param verify:
        :return:
        """
        try:
            if req_data:
                self._log_info("PUT request payload: " + str(req_data))
            if req_url:
                self._log_info("PUT request url: " + req_url)
            if req_header:
                self._log_info("PUT request header: " + str(req_header))
            response = requests.request("PUT", req_url, headers=req_header, data=req_data, verify=verify)
            if response is not None:
                self._log_info(
                    "Http PUT Response: " + response.text + " and status: " + str(response.status_code))
                return response
            else:
                self._log_error("The http response is None and not receive successfully!")
        except RequestException as ex:
            log_error = "Send PUT request and get the un-expected exception RequestException:" + str(ex)
            self._log_error(log_error)
        except ConnectTimeout as ex:
            log_error = "Send PUT request and get the un-expected exception ConnectTimeout:" + str(ex)
            self._log_error(log_error)

    @Decorator.log(my_logger=TestLog())
    @Decorator.exception_handler()
    def do_patch(self, req_url=None, req_data=None, req_header=None, verify=False):
        """

        :param req_url:
        :param req_data:
        :param req_header:
        :param verify:
        :return:
        """
        try:
            if req_data:
                self._log_info("Patch request payload: " + str(req_data))
            if req_url:
                self._log_info("Patch request url: " + req_url)
            if req_header:
                self._log_info("Patch request header: " + str(req_header))
            response = requests.request("PATCH", req_url, headers=req_header, data=req_data, verify=verify)
            if response is not None:
                self._log_info(
                    "Http PATCH Response: " + response.text + " and status: " + str(response.status_code))
                return response
            else:
                self._log_error("The http response is None and not receive successfully!")
        except RequestException as ex:
            log_error = "Send PATCH request and get the un-expected exception RequestException:" + str(ex)
            self._log_error(log_error)
        except ConnectTimeout as ex:
            log_error = "Send PATCH request and get the un-expected exception ConnectTimeout:" + str(ex)
            self._log_error(log_error)

    @Decorator.log(my_logger=TestLog())
    @Decorator.exception_handler()
    def do_delete(self, req_url=None, req_data=None, req_header=None):
        """

        :param req_url:
        :param req_data:
        :param req_header:
        :return:
        """
        try:
            if req_data:
                self._log_info("DELETE request payload: " + str(req_data))
            if req_url:
                self._log_info("DELETE request url: " + req_url)
            if req_header:
                self._log_info("DELETE request header: " + str(req_header))
            response = requests.request("DELETE", req_url, headers=req_header, data=req_data, verify=False)
            if response is not None:
                self._log_info(
                    "Http DELETE Response: " + response.text + " and status: " + str(response.status_code))
                return response
            else:
                self._log_error("The http response is None and not receive successfully!")
        except RequestException as ex:
            log_error = "Send DELETE request and get the un-expected exception RequestException:" + str(ex)
            self._log_error(log_error)
        except ConnectTimeout as ex:
            log_error = "Send DELETE request and get the un-expected exception ConnectTimeout:" + str(ex)
            self._log_error(log_error)

    def _log_info(self, log_info):
        self.logger.debug(log_info)
        TestLog.attach_allure_log(
            log_message=log_info,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)

    def _log_error(self, log_error):
        self.logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=log_error,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_error)
