import os

from behave import *
from botocore.exceptions import ClientError

from common_libraries.data_generator import DataGenerator
from common_libraries.decorator import Decorator
from common_libraries.logger import TestLog


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I created a bucket with specified name {bucket_name}")
def step_impl(context, bucket_name):
    """
    Create Bucket with specified name
    :param context:
    :param bucket_name: the bucket name
    :return:
    """
    os.environ["bucket_name"] = bucket_name
    context.bucket_name = bucket_name
    logger = context.logger
    try:
        logger.debug("======> start to create the bucket " + bucket_name)
        if context.boto3_client.s3_resource_check_bucket_exists(bucket_name):
            context.s3_resource.Bucket(bucket_name).delete()
        http_status_code = context.boto3_client.s3_resource_create_bucket(bucket_name=bucket_name)
        context.actual_status_code_create_bucket = http_status_code
        TestLog.attach_allure_log(
            log_message="Create Bucket and get the expected status code: " + str(http_status_code),
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        assert int(http_status_code) == 200
    except ClientError as ex:
        if ex.response['Error']['Code'] == 'InternalError':  # Generic error
            log_error = "Fail to create bucket <" + bucket_name + "> and get the InternalError: " + str(
                ex.response)
        else:
            log_error = "Fail to create bucket <" + bucket_name + "> and get the error: " + str(
                ex.response)
        logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=log_error,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(str(log_error))


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I create a bucket on default region")
def step_impl(context):
    """
    Create Bucket on default region
    :param context:
    :return:
    """
    bucket_name = DataGenerator.generate_bucket_name(context)
    logger = context.logger
    try:
        logger.debug("======> start to create the bucket: " + bucket_name)
        http_status_code = context.boto3_client.s3_resource_create_bucket(bucket_name=bucket_name)
        context.actual_status_code_create_bucket = http_status_code
        TestLog.attach_allure_log(
            log_message="Create Bucket and get th expected status code: " + str(http_status_code),
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        logger.debug(
            "======> end to create the bucket <" + str(bucket_name) + "> with status code：" + str(http_status_code))
        assert int(http_status_code) == 200
    except ClientError as ex:
        log_error = "Fail to create bucket <" + bucket_name + "> caused by the exception " + str(
            ex)
        logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=log_error,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_error)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I created a bucket with prefix {bucket_prefix}")
def step_impl(context, bucket_prefix):
    """
    Create Bucket with specific prefix
    :param context:
    :param bucket_prefix:
    :return:
    """
    bucket_name = DataGenerator.generate_bucket_name(context, bucket_prefix=bucket_prefix)
    logger = context.logger
    try:
        logger.debug("======> start to create the bucket " + bucket_name)
        http_status_code = context.boto3_client.s3_resource_create_bucket(bucket_name=bucket_name)
        TestLog.attach_allure_log(
            log_message="Create bucket's status: " + str(http_status_code),
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        context.actual_status_code_create_bucket = http_status_code
        assert int(http_status_code) == 200
    except ClientError as ex:
        log_error = "Fail to create bucket with prefix <" + bucket_name + "> caused by the exception " + str(
            ex)
        logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=log_error,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_error)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I get bucket location")
def step_impl(context):
    """
    Create Bucket on default region
    :param context:
    :return:
    """
    bucket_name = context.bucket_name
    logger = context.logger
    try:
        if context.boto3_client.s3_waiters_check_bucket_exists(bucket_name=bucket_name):
            get_bucket_location_response = context.s3_client.get_bucket_location(Bucket=bucket_name)
            http_status_code = get_bucket_location_response["ResponseMetadata"]["HTTPStatusCode"]
            expected_status_code = "200"
            assert int(
                http_status_code) == int(
                expected_status_code), f"The excepted status code '" + expected_status_code + "'  but the actual status code '" + str(
                http_status_code) + "'"
            TestLog.attach_allure_log(
                log_message="Get Bucket Location: " + str(get_bucket_location_response),
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        else:
            log_error = "Bucket '" + bucket_name + "' is not found as expected!"
            TestLog.attach_allure_log(
                log_message=log_error,
                log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
            raise Exception(log_error)
    except ClientError as ex:
        log_error = "Fail to Get Bucket Location for bucket <" + bucket_name + "> caused by the exception " + str(
            ex)
        logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=log_error,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_error)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I list all objects")
def step_impl(context):
    """
    List all object in the bucket
    :param context:
    :return:
    """
    bucket_name = context.bucket_name
    logger = context.logger
    try:
        bucket = context.s3_resource.Bucket(bucket_name)
        if context.boto3_client.s3_waiters_check_bucket_exists(bucket_name=bucket_name):
            objects = bucket.objects.all()
            context.logger.debug("======> List all objects of bucket: " + str(bucket_name))
            for obj in objects:
                context.logger.debug("Object key: " + str(obj.key))
            TestLog.attach_allure_log(
                log_message="List bucket objects: " + str(objects),
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        else:
            log_error = "Bucket '" + bucket_name + "' is not found as expected!"
            TestLog.attach_allure_log(
                log_message=log_error,
                log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
            raise Exception(log_error)
    except ClientError as ex:
        log_error = "Fail to list all objects for bucket <" + bucket_name + "> caused by the exception " + str(
            ex)
        logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=log_error,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_error)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I verify create bucket status {expected_status_code_create_bucket}")
def step_impl(context, expected_status_code_create_bucket):
    """
    Check bucket creation result
    :param context:
    :param expected_status_code_create_bucket:
    :return:
    """
    logger = context.logger
    if str(expected_status_code_create_bucket) == str(context.actual_status_code_create_bucket):
        log_message = "Bucket created successfully with expected status code: " + str(
            expected_status_code_create_bucket)
        logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    else:
        log_error = "Fail to verify bucket creation status, actual status code '" + str(
            context.actual_status_code_create_bucket) + "' and expected status code '" + str(
            expected_status_code_create_bucket) + "' "
        logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=log_error,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_error)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I verify delete bucket status {expected_status_code_delete_bucket}")
def step_impl(context, expected_status_code_delete_bucket):
    """
    Check bucket deletion result
    :param context:
    :param expected_status_code_delete_bucket:
    :return:
    """
    if str(expected_status_code_delete_bucket) == str(context.actual_status_code_delete_bucket):
        log_message = "Bucket deleted successfully with expected status code: " + str(
            expected_status_code_delete_bucket)
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    else:
        log_error = "Bucket deleted failed with status code " + str(context.actual_status_code_delete_bucket)
        TestLog.attach_allure_log(
            log_message=log_error,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_error)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I delete the bucket")
def step_impl(context):
    """
    Delete the bucket
    :param context:
    :return:
    """
    bucket_name = context.bucket_name
    try:
        if context.boto3_client.s3_resource_check_bucket_exists(bucket_name=bucket_name):
            http_status_code = context.boto3_client.s3_delete_bucket(bucket_name=bucket_name)
            context.actual_status_code_delete_bucket = http_status_code
            log_message = "Bucket deleted successfully: " + bucket_name + " And status code is " + str(http_status_code)
            context.logger.debug(log_message)
            TestLog.attach_allure_log(
                log_message=log_message,
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            assert int(http_status_code) == 204
        else:
            log_message = "Bucket '" + bucket_name + "' not exist in the system : "
            context.logger.debug(log_message)
            context.actual_status_code_delete_bucket = 204
            TestLog.attach_allure_log(
                log_message=log_message,
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    except ClientError as ex:
        log_error = "Fail to Delete bucket '" + bucket_name + "' and caused by the exception: " + str(ex)
        TestLog.attach_allure_log(
            log_message=log_error,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_error)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I delete the object with key {object_key}")
def step_impl(context, object_key):
    """
    Delete object with specified key
    :param context:
    :param object_key:
    :return:
    """
    # delete object
    bucket_name = context.bucket_name
    context.logger.debug("======> start to delete object of bucket  " + bucket_name)
    if context.boto3_client.s3_check_if_file_exist(bucket_name=bucket_name, file=object_key):
        if context.boto3_client.s3_waiters_check_bucket_exists(bucket_name=bucket_name):
            if context.boto3_client.s3_waiters_check_object_exists(bucket_name=bucket_name, key_name=object_key):
                obj = context.s3_resource.Object(bucket_name, object_key)
                delete_object_response = obj.delete()
                if delete_object_response:
                    log_message = "Delete object [" + object_key + "] from the Bucket: " + bucket_name
                    context.logger.debug(log_message)
                    TestLog.attach_allure_log(
                        log_message=log_message,
                        log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
                    context.logger.debug("delete object: " + str(delete_object_response))
                    context.actual_status_code_delete_bucket = delete_object_response['ResponseMetadata'][
                        'HTTPStatusCode']
                    context.logger.debug("======> end to delete object of bucket")
                    context.logger.debug(
                        "delete object from bucket '" + bucket_name + "': " + str(delete_object_response))
            else:
                log_message = "Object not found from the Bucket: " + bucket_name
                context.logger.debug(log_message)
                TestLog.attach_allure_log(
                    log_message=log_message,
                    log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    else:
        TestLog.attach_allure_log(
            log_message="Object is not found: " + bucket_name,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I delete the bucket with name {bucket_name}")
def step_impl(context, bucket_name):
    """
    Delete bucket with specified name
    :param context:
    :param bucket_name:
    :return:
    """
    context.logger.debug("======> start to delete the bucket")
    if context.boto3_client.s3_resource_check_bucket_exists(bucket_name=bucket_name):
        response = context.s3_resource.Bucket(bucket_name).delete()
        context.logger.debug("delete bucket's response: " + str(response))
        context.logger.debug("======> end to delete the bucket : " + bucket_name)
        log_message = "Delete the bucket: " + bucket_name
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
