from behave import *
from botocore.exceptions import ClientError

from common_libraries.data_generator import DataGenerator
from common_libraries.decorator import Decorator
from common_libraries.logger import TestLog


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I update bucket's versioning status {versioning_status}")
def step_impl(context, versioning_status):
    """
    Update Bucket Versioning Status
    :param context:
    :param versioning_status:
    :return:
    """
    bucket_name = context.bucket_name
    try:
        if versioning_status == "Enable":
            response = context.s3_resource.BucketVersioning(bucket_name).enable()
        elif versioning_status == "Disable":
            response = context.s3_resource.BucketVersioning(bucket_name).disable()
        else:
            response = context.s3_resource.BucketVersioning(bucket_name).put(
                VersioningConfiguration={
                    'Status': versioning_status
                }
            )
        if response:
            log_message = "Update the bucket's versioning status successfully for the bucket'" + bucket_name + "' "
            context.logger.debug(log_message)
            TestLog.attach_allure_log(
                log_message=log_message,
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)

    except ClientError as ex:
        if 'Illegal argument value: Status' in str(ex):
            context.logger.debug("invalid argument found as expected")
        else:
            raise Exception("Fail to change bucket versioning status: " + versioning_status)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I enable bucket's versioning")
def step_impl(context):
    bucket_name = context.bucket_name
    bucket_versioning = context.s3_resource.BucketVersioning(bucket_name)
    enable_bucket_versioning_response = bucket_versioning.enable()
    context.logger.debug("Enable Bucket Versioning status: " + str(enable_bucket_versioning_response))
    if context.boto3_client.s3_resource_check_bucket_exists:
        if bucket_versioning.status:
            log_message = "Get Bucket Versioning status : " + str(
                context.s3_resource.BucketVersioning(bucket_name).status + " for the bucket: " + bucket_name)
        else:
            # wait for some time and re-check bucket versioning status
            if context.boto3_client.s3_resource_check_bucket_exists:
                log_message = "Get Bucket Versioning status : " + str(
                    context.s3_resource.BucketVersioning(bucket_name).status + " for the bucket: " + bucket_name)
            else:
                log_message = "Bucket versioning updated by status can't get !"
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I disable bucket's versioning status")
def step_impl(context):
    """
    Disable Bucket Versioning Status
    :param context:
    :return:
    """
    bucket_name = context.bucket_name
    response = context.s3_resource.BucketVersioning(bucket_name).suspend()
    context.logger.debug("Disable bucket versioning status: " + str(response))
    log_message = "Disable bucket versioning status : " + str(
        context.s3_resource.BucketVersioning(bucket_name).status + " for the bucket: " + bucket_name)
    context.logger.debug(log_message)
    TestLog.attach_allure_log(
        log_message=log_message,
        log_type=TestLog.LOG_TYPE_TEST_EXECUTION)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I put bucket's versioning {versioning_status} with matched content-MD5")
def step_impl(context, versioning_status):
    """
    Put Bucket Versioning with content-MD5
    :param context:
    :param versioning_status:
    :return:
    """
    contents = '<VersioningConfiguration xmlns="http://s3.amazonaws.com/doc/2006-03-01/"><Status>' + versioning_status + '</Status></VersioningConfiguration>'
    contents_md5 = DataGenerator().calculate_content_md5(contents)
    bucket_name = context.bucket_name
    response = context.s3_resource.BucketVersioning(bucket_name).put(
        VersioningConfiguration={
            'Status': versioning_status
        },
        ContentMD5=contents_md5
    )
    context.logger.debug("Enable versioning status: " + str(response))
    log_message = "Enable bucket versioning status : " + str(
        context.s3_resource.BucketVersioning(bucket_name).status + " for the bucket: " + bucket_name)
    context.logger.debug(log_message)
    TestLog.attach_allure_log(
        log_message=log_message,
        log_type=TestLog.LOG_TYPE_TEST_EXECUTION)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I put bucket's versioning {invalid_status} with mismatched content-MD5")
def step_impl(context, invalid_status):
    contents = '<VersioningConfiguration xmlns="http://s3.amazonaws.com/doc/2006-03-01/"><Status>' + invalid_status + '</Status></VersioningConfiguration>'
    contents_md5 = DataGenerator().calculate_content_md5(contents)
    bucket_name = context.bucket_name
    try:
        context.s3_resource.BucketVersioning(bucket_name).put(
            VersioningConfiguration={
                'Status': 'Enabled'
            },
            ContentMD5=contents_md5
        )
    except ClientError as ex:
        context.error = str(ex)
        if "The Content-MD5 you specified did not match what we received." in str(ex):
            TestLog.attach_allure_log(
                log_message="Put Bucket Versioning and get the expected error message: " + str(ex),
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        else:
            raise Exception("The expected exception is not returned as expected in the response: " + str(ex))


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I put bucket's versioning {versioning_status} with MFA {mfa}")
def step_impl(context, versioning_status, mfa):
    """
    Put bucket versioning with MFA
    :param context:
    :param versioning_status:
    :param mfa:
    :return:
    """
    bucket_name = context.bucket_name
    response = context.s3_resource.BucketVersioning(bucket_name).put(
        VersioningConfiguration={
            'Status': versioning_status
        },
        MFA=mfa
    )
    context.logger.debug("Put versioning status with MFA: " + str(response))
    log_message = "Put versioning status with MFA : " + str(
        context.s3_resource.BucketVersioning(bucket_name).status + " for the bucket: " + bucket_name)
    context.logger.debug(log_message)
    TestLog.attach_allure_log(
        log_message=log_message,
        log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
