"""
Python utility to execute behave based tests in parallel mode
Usage:
python ose_test_runner.py -p=<num_of_process> -t=<bdd_tag> -F=<features_location>
Default Values:
num_of_process = 5
features_location = reusable_bdd_ui/tests/features/
if tags are not provided, all the features will be executed
if tags are provided, feature containing tagged scenarios will be executed
As of now, only feature level parallelization is supported and not scenario level.
"""
import argparse
import json
import logging
import os
import pathlib
import random
import shutil
import smtplib
import time
from email.header import Header
from email.mime.application import MIMEApplication
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from functools import partial
from multiprocessing import Pool
from os.path import basename
from subprocess import call, Popen, PIPE
from zipfile import ZipFile

from bs4 import BeautifulSoup
from configobj import ConfigObj

from common_libraries.boto3_client import Boto3Client
from common_libraries.data_providers.file_utils import FileUtils
from common_libraries.helper import Helper
from common_libraries.logger import TestLog
from testbase.api_testbase import ApiTestBase

logging.basicConfig(level=logging.INFO,
                    format="[%(levelname)-8s %(asctime)s] %(message)s")


class TestRunner:
    test_runner_logger = TestLog().set_logger()

    @staticmethod
    def parse_arguments():
        """
        Parses commandline arguments
        :return: Parsed arguments
        """
        current_path = os.getcwd()
        parser = argparse.ArgumentParser('Run Behave Scenarios')
        parser.add_argument('--allure_reporting', help='Please specify If you want to generate allure reports or not',
                            default="true")
        parser.add_argument('--html_reporting', help='Please specify If you want to generate html reports or not',
                            default="true")
        parser.add_argument('--junit_reporting', help='Please specify If you want to generate junit reports or not',
                            default="false")
        parser.add_argument('--junit', '-j', action='store_true', help='Junit report')
        parser.add_argument('--junit_directory', '-J', help='Junit report location', default='reports/')
        parser.add_argument('--processes', '-p', type=int, help='Please set Maximum number of processes. Default = 5',
                            default=10)
        parser.add_argument('--define', '-D', action='append',
                            help='Define user-specific data for the config.userdata dictionary')
        parser.add_argument("--project_name", "-pro_name", required=True, default="common_test_suite",
                            help="Please set the project/working directory to run the test suite/features")
        parser.add_argument('--features', '-F', help='Please specify behave feature to run',
                            default=os.path.join(current_path, 'common_test_suite', 'features'))
        parser.add_argument('--excludefeatures', '-EF',
                            help='Please specify excluded behave feature which should be skipped',
                            default="")
        parser.add_argument('--tags', '-t', required=False, help='Please specify behave tags to run')
        parser.add_argument('--outfile_prefix', '-o', required=False, help='Please specify outfile prefix to run')
        parser.add_argument("--env_file", "-env", required=True, help='Please specify the yaml file to run the test')
        parser.add_argument("--tags_exclude", "-te", required=False,
                            help='Please specify behave tags which exclude from the test execution')
        parser.add_argument("--test_data_file", "-test_data", required=False)
        parser.add_argument("--logging_level", "-log_level", required=False, help='Please set the logging level',
                            default="INFO")
        parser.add_argument("--tenant_user_password", "-userpasswd", required=False,
                            help='Please set tenant user\'s password')
        parser.add_argument("--test_configuration", "-test_config", required=False,
                            help='Please provider test configuration json file')
        # Provide arguments to send email test report
        parser.add_argument("--smtp_server", "-smtp", required=False, help='Please set the smtp server')
        parser.add_argument("--report_sender", "-sender", required=False, help='Please set report sender')
        parser.add_argument("--report_receiver", "-receiver", required=False, help='Please set the report receiver')
        # Provide VCD endpoint and OSE endpoint
        parser.add_argument("--vcd_endpoint", "-vcd", required=True, help='Please set the VCD Endpoint')
        parser.add_argument("--ose_endpoint", "-ose", required=True, help='Please set the OSE Endpoint')
        parser.add_argument("--bucket_prefix", "-bkt", required=False, help='Please set the Bucket name prefix',
                            default='ose-api-bkt')
        parser.add_argument("--test_report_directory", "-TD", help="Allure Test Report location")
        parser.add_argument("--test_log_directory", "-TL", help='Allure Test Log location')

        args = parser.parse_args()
        return args

    @staticmethod
    def _run_parallel_feature(feature, tags=None, userdata=None, report_loc=None, args=None):
        """
        Runs features matching given tags and userdata
        :param feature: Feature that should be run
        :type feature: str
        :param tags: Tags features should contain
        :type tags: str
        :param userdata: Userdata for behave
        :type userdata: list
        :return: Feature and status
        """

        feature_name = feature[feature.rindex("/") + 1:]
        TestRunner.test_runner_logger.info("======> Processing feature: {}".format(feature_name) + " <======")
        test_report_directory = os.environ["test_report_directory"]
        ts = time.strftime("%Y-%m-%d-%H-%M-%S") + "-" + str(random.randint(0, 100))
        html_test_report_file = os.path.join(test_report_directory,
                                             "automation_report_" + feature_name + "_" + str(ts) + ".html")
        args.html_report_directory = html_test_report_file
        allure_results_directory = args.allure_results_directory
        args.test_report_args = "-f html -o " + html_test_report_file + " -f allure_behave.formatter:AllureFormatter -o " + allure_results_directory
        params = ""
        if userdata:
            params = ""
            params = params + " {0} --no-capture".format(
                ' -D '.join(userdata))
        if report_loc and str(args.junit_reporting).lower() == 'true':
            params = ""
            params = params + " --junit --junit-directory {}".format(
                report_loc)
        if str(args.html_reporting).lower() == 'true':
            params = ""
            params = params + " -f html - o {}".format(
                args.html_report_directory)
        if str(args.allure_reporting).lower() == 'true':
            params = ""
            if "test_report_args" in args:
                params = params + args.test_report_args
        # include tags
        if args.tags:
            params = params + " -t " + args.tags
        # exclude tags
        if args.tags_exclude:
            params = params + " -t=~" + args.tags_exclude

        cmd = "behave {0} {1}".format(params, feature)
        TestRunner.test_runner_logger.debug(
            "======> Running Behave command for parallel execution: {}".format(cmd) + " <======")
        r = call(cmd, shell=True)
        status = 'ok' if r == 0 else 'failed'
        return feature, status, html_test_report_file

    @staticmethod
    def _run_sequential(args):
        """
        This uses to execute the test scenario sequentially
        """
        if os.path.exists("reports"):
            shutil.rmtree("reports")
        if os.path.exists("allure-reports"):
            shutil.rmtree("allure-reports")
        project_path = os.path.join(os.environ["project_directory"])
        feature_path = os.path.join(project_path, "features")
        params = ""
        feature_files = []
        if args.define:
            params = ""
            params = params + " -D {0} --no-capture".format(' -D '.join(args.define))
        if str(args.junit_reporting).lower() == 'true' and args.junit_directory:
            params = ""
            params = params + " --junit --junit-directory {}".format(args.junit_directory)
        if str(args.html_reporting).lower() == 'true':
            params = ""
            params = params + " -f html -o {}".format(args.html_report_directory)
        if str(args.allure_reporting).lower() == 'true':
            params = ""
            if "test_report_args" in args:
                params = params + args.test_report_args

        # include tags
        if args.tags:
            params = params + " -t " + args.tags
        # exclude tags
        if args.tags_exclude:
            params = params + " -te = ~" + args.tags_exclude
        if args.tags is None:
            if args.features and os.path.exists(args.features):
                # case 1: by default, run all features
                for root, dirs, files in os.walk(feature_path):
                    for file in files:
                        if file.endswith(".feature"):
                            feature_files.append(os.path.join(root, file))
            else:
                # case 2: run specific feature
                feature_files = []
                os.chdir(feature_path)
                for root, dirs, files in os.walk(feature_path):
                    for file in files:
                        if file.endswith(".feature"):
                            file_name = os.path.basename(file)
                            if file_name == str(args.features) or file_name in str(args.features):
                                feature_files.append(os.path.join(root, file))
            features = feature_files
            TestRunner.test_runner_logger.info("Plan to run {} features. ".format(len(features)))
            feature_index = 1
            for feature in features:
                TestRunner.test_runner_logger.info(
                    "======> Running feature[" + str(feature_index) + "] ': " + feature + "' <======")
                feature_index = feature_index + 1
            if isinstance(features, list):
                for feature in features:
                    cmd = 'behave {0} {1}'.format(params, feature)
                    TestRunner.test_runner_logger.debug(
                        "======> Executing scenarios in sequential mode: {}".format(cmd) + " <======")
                    r = call(cmd, shell=True)
                    status = 'ok' if r == 0 else 'failed'
                    TestRunner.test_runner_logger.info("Sequential Execution Status:" + status)
            else:
                cmd = 'behave {0} {1}'.format(params, features)
                TestRunner.test_runner_logger.debug(
                    "======> Executing scenarios in sequential mode: {}".format(cmd) + " <======")
                r = call(cmd, shell=True)
                status = 'ok' if r == 0 else 'failed'
                TestRunner.test_runner_logger.info("Sequential Execution Status:" + status)
        else:
            cmd = 'behave {0} {1}'.format(params, feature_path)
            TestRunner.test_runner_logger.debug(
                "======> Executing scenarios in sequential mode: {}".format(cmd) + " <======")
            r = call(cmd, shell=True)
            status = 'ok' if r == 0 else 'failed'
            TestRunner.test_runner_logger.info("Sequential Execution Status:" + status)

    @staticmethod
    def _run_parallel(args, pool):
        """
        This uses to execute the features parallel
        """
        if os.path.exists("reports"):
            shutil.rmtree("reports")
        if os.path.exists("allure-reports"):
            shutil.rmtree("allure-reports")
        TestRunner.test_runner_logger.info("======> Executing suite in parallel <======")
        file_path = os.path.join(os.getcwd(), os.environ["project_directory"])
        if args.tags:
            p = Popen('behave "' + args.features + '" -d -f json --no-summary -t ' + args.tags, stdout=PIPE, shell=True)
            out, err = p.communicate()
            j = json.loads(out)

            features = [e['location'].replace(args.features, '')[:-2] for e in j if e['status'] == 'untested']

        else:
            feature_files = []
            feature_file_path = os.path.join(file_path, "features")
            os.chdir(feature_file_path)
            if args.features:
                if args.features and os.path.exists(args.features):
                    for root, dirs, files in os.walk(args.features):
                        for file in files:
                            if file.endswith(".feature"):
                                feature_files.append(os.path.join(root, file))
                else:
                    for root, dirs, files in os.walk(feature_file_path):
                        for file in files:
                            if file.endswith(".feature"):
                                file_name = os.path.basename(file)
                                if file_name == str(args.features) or file_name in str(args.features):
                                    feature_files.append(os.path.join(root, file))
            else:
                for root, dirs, files in os.walk(feature_file_path):
                    for file in files:
                        if file.endswith(".feature"):
                            feature_files.append(os.path.join(root, file))
            features = feature_files

        # parse excluded features from external parameters
        if args.excludefeatures:
            TestRunner.test_runner_logger.info(f"Skipped features {args.excludefeatures}")
            for root, dirs, files in os.walk(feature_file_path):
                for file in files:
                    if file.endswith(".feature"):
                        file_name = os.path.basename(file)
                        if file_name in str(args.excludefeatures).replace("\"", ""):
                            item_index = Helper.get_item_index_from_list(features, file_name)
                            TestRunner.test_runner_logger.info(f"Skipped feature {file_name}")
                            del feature_files[item_index]
            features = feature_files
        # skip exclude features from yaml file
        if "vcd-site_exclude_feature" in os.environ:
            exclude_file_list = eval(os.environ["vcd-site_exclude_feature"])
            if exclude_file_list:
                for file in exclude_file_list:
                    if str(file) in str(features):
                        item_index = Helper.get_item_index_from_list(features, file)
                        del features[item_index]
        feature_to_run = len(features)
        os.environ["feature_to_run"] = str(feature_to_run)
        if feature_to_run >= 1:
            TestRunner.test_runner_logger.info(f"Features list: {str(features)}.")
            TestRunner.test_runner_logger.info("Plan to run {} features. ".format(len(features)))

            feature_index = 1
            for feature in features:
                TestRunner.test_runner_logger.info(
                    "======> Running feature[" + str(feature_index) + "]: " + feature + "' <======")
                feature_index = feature_index + 1
            run_feature = partial(TestRunner._run_parallel_feature, tags=args.tags, userdata=args.define,
                                  report_loc=args.junit_directory, args=args)
            features = [f for f in features if 'aft' not in f]
            for feature, status, html_test_report_file in pool.map(run_feature, features):
                os.environ["html_test_report_file"] = html_test_report_file
        else:
            TestRunner.test_runner_logger.info("Plan to run {} features. ".format(len(features)))

    @staticmethod
    def _set_test_directory(project_name=None, test_args=None):

        # ts store timestamp of current time
        ts = time.strftime("%Y-%m-%d-%H-%M-%S")
        # get parent directory
        parent_directory = os.getcwd()
        project_directory = os.path.join(parent_directory, project_name)
        os.environ["project_directory"] = project_directory
        # Set test data directory
        test_data_directory = os.path.join(project_directory, "test_data")
        os.environ["test_data_directory"] = test_data_directory
        # Set test environment directory
        test_environment_directory = os.path.join(project_directory, "test_environment")
        os.environ["test_environment_directory"] = test_environment_directory

        test_templates_directory = os.path.join(project_directory, "templates")
        os.environ["test_templates_directory"] = test_templates_directory

        if "env_file" in test_args:
            test_environment_yaml = str(os.path.join(test_environment_directory, test_args.env_file)).replace("\"", "")
            os.environ["test_environment_yaml"] = test_environment_yaml

        # Set test log directory
        if "test_log_directory" in test_args and test_args.test_log_directory is not None:
            test_log_directory = test_args.test_log_directory
            os.environ["test_log_directory"] = test_log_directory
        else:
            test_log_directory = os.path.join(project_directory, "test_log")
            os.environ["test_log_directory"] = test_log_directory
        if "test_report_directory" in test_args and test_args.test_report_directory is not None:
            test_report_directory = test_args.test_report_directory
        else:
            test_report_directory = os.path.join(project_directory, "test_report", time.strftime("%Y-%m-%d"), ts)
        print("test_log_directory: " + test_log_directory)
        print("test_report_directory: " + test_report_directory)

        # Set test report directory
        zip_file_directory = os.path.join(test_report_directory, "zip_test_report")
        os.environ["zip_file_directory"] = zip_file_directory
        if not os.path.exists(test_report_directory):
            os.makedirs(test_report_directory)
        #         if not os.path.exists(zip_file_directory):
        #             os.makedirs(zip_file_directory)
        test_report_file = os.path.join(test_report_directory, "ose_api_test_" + str(ts) + ".html")
        os.environ["test_report_file"] = test_report_file
        os.environ["test_report_directory"] = test_report_directory
        # feature file directory
        if "features" in test_args:
            os.environ["feature_files_parent_directory"] = test_args.features

    @staticmethod
    def _set_global_variables(test_args):
        # set bucket prefix
        if "bucket_prefix" in test_args:
            os.environ["bucket_prefix"] = test_args.bucket_prefix
        # set logging level
        if "logging_level" in test_args:
            logging_level = test_args.logging_level
            os.environ["logging_level"] = logging_level
        else:
            # by default, using logging level as "INFO"
            os.environ["logging_level"] = "INFO"
        if "vcd_endpoint" in test_args and test_args.vcd_endpoint:
            vcd_endpoint_url = str(test_args.vcd_endpoint).replace("\"", "")
            os.environ["vcd-site_vcd_endpoint"] = vcd_endpoint_url
        else:
            raise Exception("Please provide the Cloud Director URL")
        if "ose_endpoint" in test_args and test_args.ose_endpoint:
            ose_endpoint_url = str(test_args.ose_endpoint).replace("\"", "")
            os.environ["vcd-site_ose_url"] = ose_endpoint_url
        else:
            raise Exception("Please provide the OSE endpoint URL")

    @staticmethod
    def _data_preparation(test_args=""):
        api_test_base = ApiTestBase()
        test_environment_yaml = str(os.environ["test_environment_yaml"]).replace("\"", "")
        if test_args:
            if "vcd_endpoint" in test_args:
                vcd_endpoint_url = str(test_args.vcd_endpoint).replace("\"", "")
                os.environ["vcd-site_vcd_endpoint"] = str(vcd_endpoint_url).replace("\"", "")
                FileUtils.update_data_in_yaml_file(
                    yaml_file_path=test_environment_yaml,
                    parameter_name="vcd_endpoint",
                    parameter_value=vcd_endpoint_url)
            if "ose_endpoint" in test_args:
                ose_endpoint_url = str(test_args.ose_endpoint).replace("\"", "")
                FileUtils.update_data_in_yaml_file(
                    yaml_file_path=test_environment_yaml,
                    parameter_name="ose_url",
                    parameter_value=ose_endpoint_url)
            if "test_configuration" in test_args and test_args.test_configuration is not None:
                ose_test_configuration = test_args.test_configuration
                ose_test_configuration_dict = json.loads(ose_test_configuration)
                # tenant one
                tenant_one_dict = ose_test_configuration_dict["tenant_one"]
                tenant1_tenant_name_test_config = tenant_one_dict["tenant_name"]
                tenant1_user_tu1 = tenant_one_dict["tenant_users"]["tu1"]
                tenant1_user_tu1_user_name = tenant1_user_tu1["username"]
                tenant1_user_tu1_user_password = tenant1_user_tu1["password"]
                tenant1_user_tu2 = tenant_one_dict["tenant_users"]["tu2"]
                tenant1_user_tu2_user_name = tenant1_user_tu2["username"]
                tenant1_user_tu2_user_password = tenant1_user_tu1["password"]
                # tenant two
                tenant_two_dict = ose_test_configuration_dict["tenant_two"]
                tenant2_tenant_name_test_config = tenant_two_dict["tenant_name"]
                tenant2_user_tu1 = tenant_two_dict["tenant_users"]["tu1"]
                tenant2_user_tu1_user_name = tenant2_user_tu1["username"]
                tenant2_user_tu1_user_password = tenant1_user_tu1["password"]
                tenant2_user_tu2 = tenant_two_dict["tenant_users"]["tu2"]
                tenant2_user_tu2_user_name = tenant2_user_tu2["username"]
                tenant2_user_tu2_user_password = tenant1_user_tu1["password"]

        FileUtils.load_yaml_with_full_loader(yaml_file_path=test_environment_yaml)
        feature_file_list = Helper.get_all_feature_files(path=os.environ["project_directory"])
        if "vcd-site_exclude_feature" in os.environ:
            exclude_file_list = os.environ["vcd-site_exclude_feature"]
            # remove exclude features
            if exclude_file_list:
                for file in eval(exclude_file_list):
                    if file in feature_file_list:
                        feature_file_list.remove(file)
        FileUtils.update_data_in_yaml_file(yaml_file_path=test_environment_yaml, parameter_name="include_feature",
                                           parameter_value=feature_file_list)
        # ==================== Start to Update tenant ossauto1 =======================

        tenant1_tenant_name = "ossauto1"
        TestRunner.test_runner_logger.info(
            "======> start to update ossauto1/oss-auto-tu1 <======")
        print(
            tenant1_tenant_name + " " + tenant1_user_tu1_user_name + " " + tenant1_user_tu1_user_password + " " + vcd_endpoint_url)
        api_test_base.get_vcloud_authorization(tenant_name=tenant1_tenant_name_test_config,
                                               tenant_user_name=tenant1_user_tu1_user_name,
                                               tenant_user_password=tenant1_user_tu1_user_password,
                                               vcd_endpoint_url=vcd_endpoint_url,
                                               req_header=None)
        platform, canonical_id, ose_tenant_id, region = api_test_base.get_current_user(
            ose_endpoint_url=ose_endpoint_url)
        # get user credential details
        owner_id, access_key, secret_key = api_test_base.get_user_credentials(ose_endpoint_url=ose_endpoint_url,
                                                                              tenant_id=ose_tenant_id,
                                                                              tenant_user_name=tenant1_user_tu1_user_name)
        TestRunner.test_runner_logger.info(
            "======> end to update ossauto1/oss-auto-tu1: " + ", canonical id '" + canonical_id + "', access key '" + access_key + "', secret key '" + secret_key + "' <======")
        # update region information
        FileUtils.update_data_in_yaml_file(
            yaml_file_path=test_environment_yaml,
            parameter_name="default_region",
            parameter_value=region)

        # update tenant id
        FileUtils.update_data_in_yaml_file(
            yaml_file_path=test_environment_yaml, tenant_name=tenant1_tenant_name,
            parameter_name="tenant_id",
            parameter_value=ose_tenant_id)
        # update user credential details
        FileUtils.update_data_in_yaml_file(
            yaml_file_path=test_environment_yaml, tenant_name=tenant1_tenant_name, tenant_user="tu1",
            parameter_name="canonical_id",
            parameter_value=canonical_id)
        FileUtils.update_data_in_yaml_file(
            yaml_file_path=test_environment_yaml, tenant_name=tenant1_tenant_name, tenant_user="tu1",
            parameter_name="access_key",
            parameter_value=access_key)
        FileUtils.update_data_in_yaml_file(
            yaml_file_path=test_environment_yaml, tenant_name=tenant1_tenant_name, tenant_user="tu1",
            parameter_name="secret_key",
            parameter_value=secret_key)
        FileUtils.update_data_in_yaml_file(
            yaml_file_path=test_environment_yaml, tenant_name=tenant1_tenant_name, tenant_user="tu1",
            parameter_name="username",
            parameter_value=tenant1_user_tu1_user_name)
        FileUtils.update_data_in_yaml_file(
            yaml_file_path=test_environment_yaml, tenant_name=tenant1_tenant_name, tenant_user="tu1",
            parameter_name="password",
            parameter_value=tenant1_user_tu1_user_password)

        # update tenant user: oss-auto-tu2
        TestRunner.test_runner_logger.info(
            "======> start to update ossauto1/oss-auto-tu2 <======")
        api_test_base.get_vcloud_authorization(tenant_name=tenant1_tenant_name_test_config,
                                               tenant_user_name=tenant1_user_tu2_user_name,
                                               tenant_user_password=tenant1_user_tu2_user_password,
                                               vcd_endpoint_url=vcd_endpoint_url,
                                               req_header=None)
        platform, canonical_id, ose_tenant_id, region = api_test_base.get_current_user(
            ose_endpoint_url=ose_endpoint_url)
        # get user credential details
        owner_id, access_key, secret_key = api_test_base.get_user_credentials(ose_endpoint_url=ose_endpoint_url,
                                                                              tenant_id=ose_tenant_id,
                                                                              tenant_user_name=tenant1_user_tu2_user_name)

        # update user credential: canonical id, access key / secret key
        FileUtils.update_data_in_yaml_file(
            yaml_file_path=test_environment_yaml, tenant_name=tenant1_tenant_name, tenant_user="tu2",
            parameter_name="canonical_id",
            parameter_value=canonical_id)
        FileUtils.update_data_in_yaml_file(
            yaml_file_path=test_environment_yaml, tenant_name=tenant1_tenant_name, tenant_user="tu2",
            parameter_name="access_key",
            parameter_value=access_key)
        FileUtils.update_data_in_yaml_file(
            yaml_file_path=test_environment_yaml, tenant_name=tenant1_tenant_name, tenant_user="tu2",
            parameter_name="secret_key",
            parameter_value=secret_key)
        FileUtils.update_data_in_yaml_file(
            yaml_file_path=test_environment_yaml, tenant_name=tenant1_tenant_name, tenant_user="tu2",
            parameter_name="username",
            parameter_value=tenant1_user_tu2_user_name)
        FileUtils.update_data_in_yaml_file(
            yaml_file_path=test_environment_yaml, tenant_name=tenant1_tenant_name, tenant_user="tu2",
            parameter_name="password",
            parameter_value=tenant1_user_tu2_user_password)
        TestRunner.test_runner_logger.info(
            "======> end to update ossauto1/oss-auto-tu2: " + ", canonical id '" + canonical_id + "', access key '" + access_key + "', secret key '" + secret_key + "' <======")

        # ==================== End to Update tenant ossauto1 =======================

        # ==================== Start to Update tenant ossauto2 =======================
        TestRunner.test_runner_logger.info(
            "======> start to update ossauto2/oss-auto-tu1 <======")
        tenant2_tenant_name = "ossauto2"
        api_test_base.get_vcloud_authorization(tenant_name=tenant2_tenant_name_test_config,
                                               tenant_user_name=tenant2_user_tu1_user_name,
                                               tenant_user_password=tenant2_user_tu1_user_password,
                                               vcd_endpoint_url=vcd_endpoint_url,
                                               req_header=None)

        platform, canonical_id, ose_tenant_id, region = api_test_base.get_current_user(
            ose_endpoint_url=ose_endpoint_url)
        # get user credential details
        owner_id, access_key, secret_key = api_test_base.get_user_credentials(ose_endpoint_url=ose_endpoint_url,
                                                                              tenant_id=ose_tenant_id,
                                                                              tenant_user_name=tenant2_user_tu1_user_name)
        TestRunner.test_runner_logger.info(
            "======> end to update ossauto2/oss-auto-tu1: " + ", canonical id '" + canonical_id + "', access key '" + access_key + "', secret key '" + secret_key + "' <======")
        # update tenant id
        FileUtils.update_data_in_yaml_file(
            yaml_file_path=test_environment_yaml, tenant_name=tenant2_tenant_name,
            parameter_name="tenant_id",
            parameter_value=ose_tenant_id)
        # update user credential: canonical id, access key / secret key
        FileUtils.update_data_in_yaml_file(
            yaml_file_path=test_environment_yaml, tenant_name=tenant2_tenant_name, tenant_user="tu1",
            parameter_name="canonical_id",
            parameter_value=canonical_id)
        FileUtils.update_data_in_yaml_file(
            yaml_file_path=test_environment_yaml, tenant_name=tenant2_tenant_name, tenant_user="tu1",
            parameter_name="access_key",
            parameter_value=access_key)
        FileUtils.update_data_in_yaml_file(
            yaml_file_path=test_environment_yaml, tenant_name=tenant2_tenant_name, tenant_user="tu1",
            parameter_name="secret_key",
            parameter_value=secret_key)
        FileUtils.update_data_in_yaml_file(
            yaml_file_path=test_environment_yaml, tenant_name=tenant2_tenant_name, tenant_user="tu1",
            parameter_name="username",
            parameter_value=tenant2_user_tu1_user_name)
        FileUtils.update_data_in_yaml_file(
            yaml_file_path=test_environment_yaml, tenant_name=tenant2_tenant_name, tenant_user="tu1",
            parameter_name="password",
            parameter_value=tenant2_user_tu1_user_password)

        # update tenant user: oss_auto-tu2
        TestRunner.test_runner_logger.info(
            "======> start to update ossauto2/oss-auto-tu2 <======")
        api_test_base.get_vcloud_authorization(tenant_name=tenant2_tenant_name_test_config,
                                               tenant_user_name=tenant2_user_tu2_user_name,
                                               tenant_user_password=tenant2_user_tu2_user_password,
                                               vcd_endpoint_url=vcd_endpoint_url,
                                               req_header=None)
        platform, canonical_id, ose_tenant_id, region = api_test_base.get_current_user(
            ose_endpoint_url=ose_endpoint_url)
        # get user credential details
        owner_id, access_key, secret_key = api_test_base.get_user_credentials(ose_endpoint_url=ose_endpoint_url,
                                                                              tenant_id=ose_tenant_id,
                                                                              tenant_user_name=tenant2_user_tu2_user_name)
        # update user credential: canonical id, access key / secret key
        FileUtils.update_data_in_yaml_file(
            yaml_file_path=test_environment_yaml, tenant_name=tenant2_tenant_name, tenant_user="tu2",
            parameter_name="canonical_id",
            parameter_value=canonical_id)
        FileUtils.update_data_in_yaml_file(
            yaml_file_path=test_environment_yaml, tenant_name=tenant2_tenant_name, tenant_user="tu2",
            parameter_name="access_key",
            parameter_value=access_key)
        FileUtils.update_data_in_yaml_file(
            yaml_file_path=test_environment_yaml, tenant_name=tenant2_tenant_name, tenant_user="tu2",
            parameter_name="secret_key",
            parameter_value=secret_key)
        FileUtils.update_data_in_yaml_file(
            yaml_file_path=test_environment_yaml, tenant_name=tenant2_tenant_name, tenant_user="tu2",
            parameter_name="username",
            parameter_value=tenant2_user_tu2_user_name)
        FileUtils.update_data_in_yaml_file(
            yaml_file_path=test_environment_yaml, tenant_name=tenant2_tenant_name, tenant_user="tu2",
            parameter_name="password",
            parameter_value=tenant2_user_tu2_user_password)
        TestRunner.test_runner_logger.info(
            "======> end to update ossauto2/oss-auto-tu2: " + ", canonical id '" + canonical_id + "', access key '" + access_key + "', secret key '" + secret_key + "' <======")
        # ==================== End to Update tenant ossauto2 =======================
        os.environ["ose_tenant_1"] = tenant1_tenant_name_test_config
        os.environ["ose_tenant_2"] = tenant2_tenant_name_test_config
        time.sleep(60)

    @staticmethod
    def _data_cleanup():
        Boto3Client().s3_delete_buckets()

    @staticmethod
    def _get_email_body_from_reported_file(file, rep_url):
        EMAIL_TMPL = """
                   <h1>{0}</h1>
                   <p class='attribute'><strong>Report Link: </strong><a href={1}>Report Link</a></p>
                   <br/>
                   """
        f = open(file, 'r+')
        mail_body = f.read()
        title_index = mail_body.find("</h1>")
        rep_title = mail_body[:title_index + 5]
        mail_smy = EMAIL_TMPL.format(rep_title,
                                     rep_url)
        new_mail_body = mail_smy + mail_body
        f.close()
        return new_mail_body

    @staticmethod
    def _send_email(rep_url=None, html_report=None, zip_file="", platform="", ose_version="", test_args=None):
        try:
            report_receivers = []
            sender = ""
            smtp_server = ""
            email_log = TestLog().set_logger(log_path=os.environ["test_log_file"])
            msg = MIMEMultipart()
            mail_subject = "[ " + platform + "] [OSE-" + ose_version + "] [Common Test Suite] API Automation Report"
            if test_args:
                if "smtp_server" in test_args:
                    smtp_server = test_args.smtp_server
                if "report_receiver" in test_args:
                    report_receivers = test_args.report_receiver
                if "report_sender" in test_args:
                    sender = test_args.report_sender
                if report_receivers:
                    report_receivers = [report_receivers]
                    report_receivers = list(set(report_receivers))
            msg['Subject'] = Header(mail_subject, 'utf-8')
            msg['From'] = sender
            msg['To'] = ", ".join(report_receivers)
            if html_report:
                new_mail_body = TestRunner()._get_email_body_from_reported_file(
                    html_report, rep_url)
                text_body = MIMEText(new_mail_body, 'html', 'utf-8')
                msg.attach(text_body)
                html_file_name = str(html_report)[html_report.rindex("/") + 1:]
                html = open(html_report)
                html_attachment = MIMEText(html.read(), 'html')
                msg.attach(html_attachment)
                # Set mail headers
                html_attachment.add_header(
                    "Content-Disposition",
                    "attachment", filename=html_file_name)
            # attach zip file
            zip_file_name = str(zip_file)[zip_file.rindex("/") + 1:]
            # open and read the file in binary
            with open(zip_file, 'rb') as file:
                # Attach the file with filename to the email
                msg.attach(MIMEApplication(file.read(), Name=zip_file_name))
            server = smtplib.SMTP(smtp_server)
            mail_text = msg.as_string()
            server.sendmail(sender, report_receivers, mail_text)
            server.quit()
            email_log.info('Test Report Email sent successfully.')

        except Exception as ex:
            raise ("Fail to send email" + str(ex))

    @staticmethod
    def _update_property_file(file_path, property_name, property_value):
        config = ConfigObj(file_path)
        config[property_name] = property_value  # Update Key
        config.write()  # Write Content

    # Zip the files from given directory that matches the filter
    @staticmethod
    def _zip_file_in_dir(dir_name, zip_file_name, filter):
        # create a ZipFile object
        with ZipFile(zip_file_name, 'w') as zipObj:
            # Iterate over all the files in directory
            for folderName, subfolders, filenames in os.walk(dir_name):
                for filename in filenames:
                    if filter(filename):
                        # create complete filepath of file in directory
                        filePath = os.path.join(folderName, filename)
                        # Add file to zip
                        zipObj.write(filePath, basename(filePath))

        zip_file = os.path.join(dir_name, zip_file_name + ".zip")
        return zip_file

    @staticmethod
    def _zip_report(file_name, file_dir, zip_file_dir):
        TestRunner.test_runner_logger.info("======> " + file_dir + " ======>")
        TestRunner.test_runner_logger.info("======> " + zip_file_dir + " ======>")
        os.chdir(zip_file_dir)
        shutil.make_archive(file_name, 'zip', file_dir)
        zip_file = os.path.join(zip_file_dir, file_name + ".zip")
        TestRunner.test_runner_logger.info("zip completed")
        return zip_file

    @staticmethod
    def _generate_html_report(file_dir, file_name):
        TestRunner.test_runner_logger.info("======> " + file_name + " ======>")
        TestRunner.test_runner_logger.info("======> " + file_dir + " ======>")
        # combine all html reports
        output_doc = BeautifulSoup()
        output_doc.append(output_doc.new_tag("html"))
        output_doc.html.append(output_doc.new_tag("head"))
        output_doc.html.append(output_doc.new_tag("body"))
        for file in os.listdir(file_dir):
            if not file.lower().endswith('.html'):
                continue
            with open(os.path.join(file_dir, file), 'r') as html_file:
                soup = BeautifulSoup(html_file.read(), "html.parser")
                html_body = soup.body
                output_doc.body.extend(html_body)
                html_head = soup.head
        output_doc.head.extend(html_head)
        all_html_report = os.path.join(file_dir, file_name + ".html")
        file_html = open(all_html_report, "w")
        file_html.write(output_doc.prettify())
        file_html.close()
        TestRunner.test_runner_logger.info("generate html report completed")
        return all_html_report

    @staticmethod
    def ose_test_runner():
        """
        Runner
        """

        platform = ""
        ose_version = ""
        # 1. Get arguments from command line
        test_args = TestRunner.parse_arguments()
        print(test_args)
        # 2. Set test related directories: project parent directory, test data/test log/report directory
        project_name = test_args.project_name
        TestRunner._set_test_directory(project_name=project_name, test_args=test_args)
        TestRunner._set_global_variables(test_args=test_args)
        project_dir = os.environ["project_directory"]
        test_log_dir = os.environ["test_log_directory"]
        # 2.a set logging level
        logging_level = os.environ["logging_level"]
        TestRunner.test_runner_logger = TestLog().set_logger(log_path=test_log_dir, logging_level=logging_level)

        # 3. Load global variables from yaml file
        test_environment_yaml = str(os.environ["test_environment_yaml"]).replace("\"", "")
        FileUtils.load_yaml_with_full_loader(yaml_file_path=test_environment_yaml)

        # 3.a update vcd endpoint and ose endpoint from external arguments
        # update VCD and OSE endpoint information
        vcd_endpoint_url = str(test_args.vcd_endpoint).replace("\"", "")
        ose_endpoint_url = str(test_args.ose_endpoint).replace("\"", "")
        FileUtils.update_data_in_yaml_file(
            yaml_file_path=test_environment_yaml,
            parameter_name="vcd_endpoint",
            parameter_value=vcd_endpoint_url)
        FileUtils.update_data_in_yaml_file(
            yaml_file_path=test_environment_yaml,
            parameter_name="ose_url",
            parameter_value=ose_endpoint_url)
        # 3.b update user's information: canonical id, access key and secret key
        TestRunner._data_preparation(test_args=test_args)
        print("Wait for YAML updated successfully")
        time.sleep(60)
        # reload global variables from yaml file
        FileUtils.load_yaml_with_full_loader(yaml_file_path=test_environment_yaml)
        # 4 Run behave test
        html_test_report_file = os.environ["test_report_file"]
        ts = time.strftime("%Y-%m-%d-%H-%M-%S")
        allure_results_directory = os.path.join(os.environ["project_directory"], "allure-results",
                                                time.strftime("%Y-%m-%d"),
                                                str(ts))
        allure_html_report_directory = os.path.join(allure_results_directory, "html-reports")
        # 4.a command args to generate html and allure report
        test_report_args = "-f html -o " + html_test_report_file + " -f allure_behave.formatter:AllureFormatter -o " + allure_results_directory
        test_args.test_report_args = test_report_args
        test_args.html_report_directory = html_test_report_file
        test_args.allure_results_directory = allure_results_directory
        # 4.b get bucket prefix to used by testing
        if "bucket_prefix" in test_args:
            os.environ["bucket_prefix"] = test_args.bucket_prefix
        else:
            os.environ["bucket_prefix"] = "ose-api-"
        # 4.c specify process and run test accordingly
        pool = Pool(test_args.processes)
        if test_args.processes > 1:
            TestRunner._run_parallel(test_args, pool)
        elif test_args.processes == 1:
            TestRunner._run_sequential(test_args)

        if "feature_to_run" in os.environ:
            feature_to_run = int(os.environ["feature_to_run"])
        else:
            feature_to_run = 1
        if feature_to_run >= 1:
            # 5 Data cleanup: delete buckets and objects
            TestRunner._data_cleanup()
            # 6 Generate allure test report
            if str(test_args.allure_reporting).lower() == 'true':
                # copy environment xml file allure report directory
                environment_xml_path = os.path.join(project_dir, "environment.properties")
                # get login credential
                ose_tenant_user_name = eval(os.environ["vcd-site_ossauto1_tu1"])["username"]
                ose_tenant_user_password = eval(os.environ["vcd-site_ossauto1_tu1"])["password"]
                # get tenant id
                ose_tenant_name = os.environ["ose_tenant_1"]
                api_test_base = ApiTestBase()
                api_test_base.get_vcloud_authorization(tenant_name=ose_tenant_name,
                                                       tenant_user_name=ose_tenant_user_name,
                                                       tenant_user_password=ose_tenant_user_password,
                                                       vcd_endpoint_url=vcd_endpoint_url,
                                                       req_header=None)
                platform, canonical_id, ose_tenant_id, region = api_test_base.get_current_user(
                    ose_endpoint_url=ose_endpoint_url)
                ose_version = api_test_base.get_ose_version(ose_endpoint_url=ose_endpoint_url)
                TestRunner._update_property_file(environment_xml_path, "Test_Platform", platform)
                TestRunner._update_property_file(environment_xml_path, "Provider_EndPoint", vcd_endpoint_url)
                TestRunner._update_property_file(environment_xml_path, "OSE_EndPoint", ose_endpoint_url)
                TestRunner._update_property_file(environment_xml_path, "OSE_Version", ose_version)
                shutil.copy2(environment_xml_path, allure_results_directory)
                categories = os.path.join(project_dir, "categories.json")
                shutil.copy2(categories, allure_results_directory)
                if "test_report_args" in test_args:
                    os.system("mkdir " + allure_html_report_directory + "")
                    allure_cmd = 'allure generate ' + allure_results_directory + ' -o ' + allure_html_report_directory
                    desktop = pathlib.Path(os.path.join(os.environ["project_directory"], "allure-results"))
                    desktop.rglob("history")
                    folder_list = list(desktop.rglob("history"))
                    if os.path.exists(allure_results_directory):
                        pass
                    else:
                        os.system("mkdir -p " + allure_results_directory)
                        TestRunner.test_runner_logger.info("======> Allure results directory created ======> ")
                    for folder in folder_list:
                        os.system("cp -r " + str(folder) + " " + allure_results_directory)
                    result = call(allure_cmd, shell=True)
                    status = 'ok' if result == 0 else 'failed'
                    TestRunner.test_runner_logger.info("======> Allure Report Generation Status:" + status + "======> ")

            TestRunner.test_runner_logger.info("======> html file " + os.environ["test_report_directory"])
            if "report_receiver" in test_args and test_args.report_receiver:
                TestRunner.test_runner_logger.info("======> start to send the test report ======>")
                TestRunner.test_runner_logger.info("======> zip files  " + os.environ["zip_file_directory"])
                html_file_name = platform + "_" + ose_version + "_" + time.strftime("%Y-%m-%d")
                zip_file_name = platform + "_" + ose_version + "_" + time.strftime("%Y-%m-%d-%H-%M-%S")
                # zip report files
                zip_file = TestRunner._zip_report(file_name=zip_file_name, file_dir=os.environ["test_report_directory"],
                                                  zip_file_dir=os.environ["zip_file_directory"])
                all_html_report = TestRunner._generate_html_report(file_dir=os.environ["test_report_directory"],
                                                                   file_name=html_file_name)
                TestRunner._send_email(
                    html_report=all_html_report,
                    rep_url="", zip_file=zip_file, platform=platform, ose_version=ose_version, test_args=test_args)
                TestRunner.test_runner_logger.info("======> end to send the test report ======>")


if __name__ == '__main__':
    my_runner = TestRunner()
    my_runner.ose_test_runner()
