import base64
import json
import os
import re

from common_libraries import http_client
from common_libraries.decorator import Decorator
from common_libraries.logger import TestLog

if "test_log_file" in os.environ:
    test_log = os.environ["test_log_file"]
    test_log_path = os.environ["test_log_directory"]
else:
    test_log_path = os.path.join(os.getcwd(), "default_log.log")
from common_libraries.http_client import HttpRequestsAPIClient


class ApiTestBase:
    http_client = None

    def __init__(self):
        self.http_client = HttpRequestsAPIClient()
        self.logger = TestLog().set_logger(log_path=os.environ["test_log_file"],
                                           logging_level=os.environ["logging_level"])

    def get_vcloud_authorization(self, tenant_name=None, tenant_user_name=None, tenant_user_password=None,
                                 vcd_endpoint_url=None,
                                 req_header=None):
        try:
            user_name = tenant_user_name + "@" + tenant_name
            user_pass = user_name + ':' + tenant_user_password
            encoded_u = base64.b64encode(user_pass.encode()).decode()
            if req_header is None:
                req_header = {
                    "Accept": "application/*+json;version=35.2,application/json;version=35.2",
                    "Authorization": "Basic %s" % encoded_u
                }

            jwt, status_code = self.http_client.do_get_vcloud_jwt(
                req_url=vcd_endpoint_url + "/api/sessions", req_header=req_header)
            # Store vCloud Authorization
            if jwt is not None:
                os.environ["Authorization"] = 'Bearer '+jwt
                self.logger.debug("Get vCloud jwt token: " + os.environ["Authorization"])
                return jwt
            else:
                raise Exception("VCD jwt token is not returned as expected!")
        except Exception as ex:
            raise Exception("VCD jwt token is not returned as expected: " + str(ex))

    def get_user_credentials(self, ose_endpoint_url=None, tenant_id=None, tenant_user_name=None):
        try:
            # get create bucket url
            url = ose_endpoint_url + "/api/v1/core/tenants/" + tenant_id + "/users/" + tenant_user_name + "/credentials"
            jwt = os.environ["Authorization"]
            req_header = {
                'Authorization': jwt
            }
            # Send GET request to retrieve credential
            response = self.http_client.do_get(req_url=url, req_header=req_header)
            response_dict = json.loads(response.text)
            if 'storageUserId' in response_dict["items"][0]:
                canonical_id = response_dict["items"][0]["storageUserId"]
            else:
                canonical_id = ""
            access_key = response_dict["items"][0]["accessKey"]
            secret_key = response_dict["items"][0]["secretKey"]
            # store status code
            os.environ["http_status_code"] = str(response.status_code)
            # return user's canonical id, access key and secret key
            return canonical_id, access_key, secret_key
        except Exception as ex:
            raise Exception("get_user_credentials() throw the exception: " + ex)

    def get_current_user(self, ose_endpoint_url=None):
        try:
            # get create bucket url
            url = ose_endpoint_url + "/api/v1/core/current-user"
            jwt = os.environ["Authorization"]
            req_header = {
                'Authorization': jwt
            }
            # Send GET request to retrieve credential
            response = self.http_client.do_get(req_url=url, req_header=req_header)
            response_dict = json.loads(response.text)
            platform = response_dict["activeRegions"][0]["platform"]
            region = response_dict["activeRegions"][0]["name"]
            # canonical_id = response_dict["id"]
            canonical_id = response_dict["storageCanonicalId"]
            tenant_id = response_dict["tenant"]["id"]
            return platform, canonical_id, tenant_id, region
        except Exception as ex:
            raise Exception("get_user_credentials() throw the exception: " + ex)

    def get_ose_version(self, ose_endpoint_url=None):
        try:
            # get create bucket url
            url = ose_endpoint_url + "/api/v1/core"
            jwt = os.environ["Authorization"]
            req_header = {
                'Authorization': jwt
            }
            # Send GET request to retrieve credential
            response = self.http_client.do_get(req_url=url, req_header=req_header)
            response_dict = json.loads(response.text)
            ose_version = response_dict["version"]
            return ose_version
        except Exception as ex:
            raise Exception("get_user_credentials() throw the exception: " + ex)

    @Decorator.log()
    def prepare_request_header(self, x_vcloud_authorization,
                               x_vmware_vcloud_tenant_context=None, tenant=None):
        # prepare API request header
        req_header = {
            'x-vmware-vcloud-auth-context': tenant,
            'Authorization': x_vcloud_authorization,
            'X-VMWARE-VCLOUD-TENANT-CONTEXT': x_vmware_vcloud_tenant_context,
            'Content-Type': 'application/json'
        }
        self.logger.info("Prepare request header: " + str(req_header))

        return req_header

    @Decorator.log()
    def create_bucket(self, req_header=None, req_payload=None, region=None,
                      ose_endpoint_url=None, bucket_name=None):
        try:
            # get create bucket url
            url = ose_endpoint_url + "/api/v1/s3/" + bucket_name
            self.logger.info("Create Bucket Url: " + url)
            # prepare payload
            if req_payload is None:
                # use the default payload to create the bucket
                payload = json.dumps({
                    "locationConstraint": region
                })

            # send put request to create the bucket
            resp = self.http_client.do_put(req_url=url, req_data=req_payload, req_header=req_header, verify=False)
            if resp.text:
                self.logger.info("Create bucket " + bucket_name + " and response " + resp.text)
            self.logger.info("Create bucket " + bucket_name + " and status code " + str(resp.status_code))
            # return status code
            os.environ["http_status_code"] = str(resp.status_code)
            return str(resp.status_code)
        except Exception as ex:
            raise Exception("create_bucket() throw the exception: " + ex)

    @Decorator.log()
    def enable_bucket_versioning(self, req_url=None, req_header=None, req_payload=None):
        try:
            # send put request to enable bucket versioning
            resp = self.http_client.do_put(req_url=req_url, req_data=req_payload, req_header=req_header, verify=False)
            if resp.text:
                self.logger.info("Enable bucket versioning response " + resp.text)
            self.logger.info("Enable bucket versioning status code " + str(resp.status_code))
            # return status code
            os.environ["http_status_code"] = str(resp.status_code)
            return str(resp.status_code)
        except Exception as ex:
            raise Exception("enable_bucket_versioning() throw the exception: " + ex)

    @Decorator.log()
    def put_bucket_tag(self, ose_endpoint_url=None, bucket_name=None, req_header=None, req_payload=None,
                       ):
        try:
            url = ose_endpoint_url + "/api/v1/s3/" + bucket_name + "?tagging"
            # prepare payload
            if req_payload is None:
                # use the default payload to add bucket bucket_tag
                req_payload = json.dumps({
                    "tagSets": [
                        {
                            "tags": [
                                {
                                    "key": "tag1",
                                    "value": "1"
                                }
                            ]
                        }
                    ]
                })
            # Put bucket bucket_tag
            resp = self.http_client.do_put(req_url=url, req_data=req_payload, req_header=req_header, verify=False)
            if resp:
                self.logger.info("Put Bucket bucket_tag of bucket <" + bucket_name + "> and response " + resp.text)
                self.logger.info(
                    "Put Bucket bucket_tag of bucket <" + bucket_name + "> and status code " + str(resp.status_code))
            # return status code
            os.environ["http_status_code"] = str(resp.status_code)
            return resp.status_code

        except Exception as ex:
            raise Exception("put_bucket_tag() throw the exception: " + ex)

    @Decorator.log()
    def put_bucket_acl(self, ose_endpoint_url=None, bucket_name=None, req_header=None, req_payload=None,
                       ):
        try:
            url = ose_endpoint_url + "/api/v1/s3/" + bucket_name + "?acl="
            # Put bucket ACL
            resp = self.http_client.do_put(req_url=url, req_data=req_payload, req_header=req_header, verify=False)
            self.logger.info("Put BucketACL of bucket <" + bucket_name + "> and response " + resp.text)
            self.logger.info(
                "Put Bucket ACL of bucket <" + bucket_name + "> and status code " + str(resp.status_code))
            # return status code
            os.environ["http_status_code"] = str(resp.status_code)
            return resp.status_code

        except Exception as ex:
            raise Exception("put_bucket_acl() throw the exception: " + ex)

    @Decorator.log()
    def put_bucket_object(self, ose_endpoint_url=None, bucket_name=None, object_name=None, req_header=None):
        try:
            url = ose_endpoint_url + "/api/v1/s3/" + bucket_name + "/" + object_name
            self.logger.info("Put Bucket Object Url: " + url)
            # Put Bucket Object
            resp = self.http_client.do_put(req_url=url, req_header=req_header, verify=False)
            if resp:
                if resp.text:
                    self.logger.info("Put Bucket Object of bucket <" + bucket_name + "> and response " + resp.text)
                self.logger.info(
                    "Put Bucket Object of bucket <" + bucket_name + "> and status code " + str(resp.status_code))
            # return status code
            os.environ["http_status_code"] = str(resp.status_code)
            return resp.status_code
        except Exception as ex:
            raise Exception("put_bucket_object() throw the exception: " + ex)

    @Decorator.log()
    def put_bucket_replication(self, req_url=None, req_header=None, req_payload=None):
        try:

            # Put bucket Replication
            resp = self.http_client.do_put(req_url=req_url, req_data=req_payload, req_header=req_header, verify=False)
            self.logger.info("Put Bucket Replication of bucket  and response " + resp.text)
            self.logger.info(
                "Put Bucket Replication and status code " + str(resp.status_code))
            # return status code
            os.environ["http_status_code"] = str(resp.status_code)
            return resp.status_code

        except Exception as ex:
            raise Exception("put_bucket_acl() throw the exception: " + ex)

    @Decorator.log()
    def get_bucket(self, ose_endpoint_url=None, bucket_name=None, req_header=None, filter_parameter=None):
        try:
            if filter_parameter:
                url = ose_endpoint_url + "/api/v1/s3/" + bucket_name + filter_parameter
            else:
                url = ose_endpoint_url + "/api/v1/s3/" + bucket_name
            # Get bucket
            resp = self.http_client.do_get(req_url=url, req_header=req_header)
            self.logger.info("Get bucket <" + bucket_name + "> and response " + resp.text)
            self.logger.info("Get bucket <" + bucket_name + "> and status code " + str(resp.status_code))
            # return status code
            os.environ["http_status_code"] = str(resp.status_code)
            return resp.status_code
        except Exception as ex:
            raise Exception("get_bucket() throw the exception: " + ex)

    @Decorator.log()
    # this is the common method to get bucket data like tag,acl,versioning,bucket location......
    def get_bucket_data(self, ose_endpoint_url=None, bucket_name=None, req_header=None, data_type=None):
        # data_type could be acl,tag,versioning,logging......
        try:
            if data_type is not None:
                url = ose_endpoint_url + "/api/v1/s3/" + bucket_name + "?" + data_type
            else:
                url = ose_endpoint_url + "/api/v1/s3/" + bucket_name
            # Get bucket data
            resp = self.http_client.do_get(req_url=url, req_header=req_header)
            self.logger.info("Get bucket <" + data_type + "> and response " + resp.text)
            self.logger.info("Get bucket <" + data_type + "> and status code " + str(resp.status_code))
            # return status code
            os.environ["http_status_code"] = str(resp.status_code)
            return resp.status_code
        except Exception as ex:
            raise Exception("get_bucket_data() throw the exception: " + ex)

    @Decorator.log()
    # this is the common method to delete bucket data like tag,acl,versioning,bucket location, bucket replication......
    def delete_bucket_data(self, ose_endpoint_url=None, bucket_name=None, req_header=None, data_type=None):
        # data_type could be acl,tag,versioning,logging......
        try:
            if data_type is not None:
                url = ose_endpoint_url + "/api/v1/s3/" + bucket_name + "?" + data_type
            else:
                url = ose_endpoint_url + "/api/v1/s3/" + bucket_name
            # Get bucket data
            resp = self.http_client.do_delete(req_url=url, req_header=req_header)
            self.logger.info("Delete bucket <" + data_type + "> and response " + resp.text)
            self.logger.info("Delete bucket <" + data_type + "> and status code " + str(resp.status_code))
            # return status code
            os.environ["http_status_code"] = str(resp.status_code)
            return resp.status_code
        except Exception as ex:
            raise Exception("delete_bucket_data() throw the exception: " + ex)

    @Decorator.log()
    def delete_bucket(self, ose_endpoint_url=None, bucket_name=None, req_header=None):
        try:
            url = ose_endpoint_url + "/api/v1/s3/" + bucket_name
            self.logger.info("Delete Bucket Url: " + url)
            # Delete bucket
            resp = self.http_client.do_delete(req_url=url, req_header=req_header)
            self.logger.info("Delete bucket < " + bucket_name + "> and response " + resp.text)
            self.logger.info("Delete bucket <" + bucket_name + "> and status code " + str(resp.status_code))
            # return status code
            os.environ["http_status_code"] = str(resp.status_code)
            return resp.status_code
        except Exception as ex:
            raise Exception("delete_bucket() throw the exception: " + ex)

    @Decorator.log()
    def delete_bucket_object(self, ose_endpoint_url=None, bucket_name=None, object_name=None, req_header=None):
        try:
            url = ose_endpoint_url + "/api/v1/s3/" + bucket_name + "/" + object_name
            self.logger.info("Delete Bucket Object Url: " + url)
            # Delete Bucket Object
            resp = self.http_client.do_delete(req_url=url, req_header=req_header)

            if resp:
                if resp.text:
                    self.logger.info("Delete Bucket Object of bucket <" + bucket_name + "> and response " + resp.text)
                self.logger.info(
                    "Delete Bucket Object of bucket <" + bucket_name + "> and status code " + str(resp.status_code))
            else:
                raise Exception("delete_bucket_object() throw the exception: " + str(resp))
            # return status code
            os.environ["http_status_code"] = str(resp.status_code)
            return resp.status_code
        except Exception as ex:
            raise Exception("delete_bucket_object() throw the exception: " + ex)

    @Decorator.log()
    def verify_http_status_code(self, actual_status_code, expected_status_code):
        try:
            if actual_status_code == expected_status_code:
                self.logger.info(
                    "Pass, Expected status code <" + str(expected_status_code) + "> is found as expected.")
                return True
            else:
                raise Exception(
                    "Fail, Expected status code is <" + str(
                        expected_status_code) + ">, but get the actual status code <" + str(
                        actual_status_code) + ">")

        except Exception as ex:
            raise Exception("verify_http_status_code() throw the exception: " + ex)

    @Decorator.log()
    def verify_http_response(self, response, expected_field, expected_value):
        try:
            if response and response.text:
                actual_response = response.text
                if expected_field in actual_response:
                    if expected_value in actual_response:
                        self.logger.info(
                            "Pass, the expected field <" + expected_field + "> and expected value <" + expected_value + "> are found as expected.")
                    else:
                        raise Exception(
                            "Fail, Expected value <" + expected_value + "> is not found in the response body!")
                else:
                    raise Exception("Fail, Expected field <" + expected_field + "> is not found in the response body!")
            else:
                raise Exception(
                    "No response is found as expected！")
        except Exception as ex:
            raise Exception("verify_http_status_code() throw the exception: " + ex)

    @Decorator.log()
    def decode_base64_data(self, data_to_encode):
        encoded_u = base64.b64encode(data_to_encode.encode()).decode()
        return str(encoded_u)

    @Decorator.log()
    def isnan(value):
        try:
            import math
            return math.isnan(float(value))
        except:
            return False

    @Decorator.log()
    def replace_data_with_global_variables(self, source_string, global_variables):
        try:
            variables_to_replace = re.findall("{{.*?}}", str(source_string))
            for var in variables_to_replace:
                variable = str(var).replace("{{", "").replace("}}", "")
                if variable in global_variables:
                    source_string = str(source_string).replace(var, global_variables[variable])

            return source_string
        except:
            pass

    @Decorator.log()
    def prepare_http_request_data(self, source_string, global_variables):
        try:
            # replace variables in test data
            test_data_dict = eval(
                self.replace_data_with_global_variables(str(source_string), global_variables))
            # 1 get http action
            if 'HttpAction' in test_data_dict:
                http_action = test_data_dict['HttpAction']
            else:
                raise Exception("Http Action can not None !")
            # 2 prepare http host
            if 'HttpHost' in test_data_dict:
                http_host = test_data_dict['HttpHost']
            else:
                raise Exception("Http Host can not None !")
            # 3 prepare http path
            if 'HttpPath' in test_data_dict:
                http_path = test_data_dict['HttpPath']
            else:
                http_path = None
            # 4 prepare http header
            req_header = {
                'Content-Type': 'application/json',
                'Authorization': os.environ["Authorization"]
            }
            # 5 prepare http request body
            if 'HttpRequestBody' in test_data_dict:
                http_request_body = test_data_dict['HttpRequestBody']
            else:
                http_request_body = None
                print("Http request body is empty.")

            http_request_url = http_host + "/" + http_path
            return http_action, http_request_url, req_header, http_request_body

        except Exception:
            raise Exception("Fail to prepare http request data")

    @Decorator.log()
    def send_http_request(self, http_action=None, http_request_url=None, http_request_body=None,
                          http_request_header=None):
        if str(http_action).lower() == "get":
            http_response = http_client.do_get(req_url=http_request_url, req_header=http_request_header)
        elif str(http_action).lower() == "post":
            http_response = http_client.do_post(req_url=http_request_url, req_data=http_request_body,
                                                req_header=http_request_header)
        elif str(http_action).lower() == "patch":
            http_response = http_client.do_patch(req_url=http_request_url, req_data=http_request_body,
                                                 req_header=http_request_header)
        elif str(http_action).lower() == "put":
            http_response = http_client.do_put(req_url=http_request_url, req_data=http_request_body,
                                               req_header=http_request_header)
        elif str(http_action).lower() == "delete":
            http_response = http_client.do_delete(req_url=http_request_url, req_header=http_request_header)
        elif str(http_action).lower() == "head":
            http_response = http_client.do_head(req_url=http_request_url, req_header=http_request_header)
        return http_response
